package gameEngine;
import javax.sound.midi.*;
import java.io.File;
import java.io.IOException;
import java.net.URL;
public class MidiPlayer {
    private Sequence seq;
    private Sequencer seqr;
    private Synthesizer synth;
    private Receiver receiver;
    private File midiFile;
    private String midiID;
    private boolean loaded;
    private boolean usingHardwareSoundbank;
    private float defaultTempo;
    public MidiPlayer() {
        loaded = false;
        try {
            seqr = MidiSystem.getSequencer();
            synth = MidiSystem.getSynthesizer();
            System.out.println ( "Setting up Midi Player..." );
            System.out.println ( "MidiDeviceInfo: " );
            for ( MidiDevice.Info info : MidiSystem.getMidiDeviceInfo() ) {
                System.out.println ( "\t" + info.getName() + ": " + info.getDescription() );
            }
            System.out.println();
            Soundbank soundbank = synth.getDefaultSoundbank();
            if ( soundbank == null ) {
                receiver = MidiSystem.getReceiver();
                usingHardwareSoundbank = true;
                System.out.println ( "using hardware soundbank" );
            } else {
                synth.loadAllInstruments ( soundbank );
                receiver = synth.getReceiver();
                usingHardwareSoundbank = false;
                System.out.println ( "using default software soundbank:" + soundbank );
            }
            seqr.getTransmitter().setReceiver ( receiver );
        } catch ( Exception e ) {
            System.out.println ( "MIDI error: It appears your system doesn't have a MIDI device or your device is not working." );
        }
    }
    public MidiPlayer ( String fileName ) {
        this();
        load ( fileName );
    }
    public void load ( String fileName ) {
        this.unload();
        try {
            URL midiURL =  getClass().getClassLoader().getResource ( fileName );
            seq = MidiSystem.getSequence ( midiURL );
            seqr.open();
            synth.open();
            seqr.setSequence ( seq );
            loaded = true;
            defaultTempo = seqr.getTempoInBPM();
        } catch ( IOException ioe ) {
            System.out.println ( "MIDI error: Problem occured while reading " + midiFile.getName() + "." );
        } catch ( InvalidMidiDataException imde ) {
            System.out.println ( "MIDI error: " + midiFile.getName() + " is not a valid MIDI file or is unreadable." );
        } catch ( Exception e ) {
            System.out.println ( "MIDI error: Unexplained error occured while loading midi." );
        }
    }
    public void unload() {
        this.stop();
        seqr.close();
        synth.close();
        midiFile = null;
        loaded = false;
    }
    public void setMidiID ( String id ) {
        midiID = id;
    }
    public String getMidiID() {
        return new String ( midiID );
    }
    public void play ( boolean reset ) {
        if ( reset ) {
            seqr.setTickPosition ( seqr.getLoopStartPoint() );
        }
        seqr.start();
    }
    public void stop() {
        if ( seqr.isOpen() ) {
            seqr.stop();
        }
    }
    public boolean isRunning() {
        return seqr.isRunning();
    }
    public float getTempo() {
        return seqr.getTempoInBPM();
    }
    public void loop ( int times ) {
        loop ( times, 0, -1 );
    }
    public void loop ( int times, long start, long end ) {
        if ( start < 0 ) {
            start = 0;
        }
        if ( end > seqr.getSequence().getTickLength() || end <= 0 ) {
            end = seqr.getSequence().getTickLength();
        }
        if ( start >= end && end != -1 ) {
            start = end - 1;
        }
        seqr.setLoopStartPoint ( start );
        seqr.setLoopEndPoint ( end );
        if ( times == -1 ) {
            seqr.setLoopCount ( Sequencer.LOOP_CONTINUOUSLY );
        } else {
            seqr.setLoopCount ( times );
        }
    }
    public void resetTempo() {
        this.changeTempo ( this.defaultTempo );
    }
    public void changeTempo ( float bpm ) {
        double lengthCoeff = bpm / seqr.getTempoInBPM();
        seqr.setLoopStartPoint ( ( long ) ( seqr.getLoopStartPoint() * lengthCoeff ) );
        seqr.setLoopEndPoint ( ( long ) ( seqr.getLoopEndPoint() * lengthCoeff ) );
        seqr.setTempoInBPM ( bpm );
    }
    public void setVolume ( double vol ) {
        System.out.println ( "Midi volume change request: " + vol );
        try {
            if ( usingHardwareSoundbank ) {
                ShortMessage volumeMessage = new ShortMessage();
                for ( int i = 0; i < 16; i++ ) {
                    volumeMessage.setMessage ( ShortMessage.CONTROL_CHANGE, i, 7, ( int ) ( vol * 127 ) );
                    receiver.send ( volumeMessage, -1 );
                }
            } else {
                MidiChannel[] channels = synth.getChannels();
                for ( int c = 0; c < channels.length; c++ ) {
                    if ( channels[c] != null ) {
                        channels[c].controlChange ( 7, ( int ) ( vol * 127 ) );
                    }
                }
            }
        } catch ( Exception e ) {
            e.printStackTrace();
        }
    }
}
