package smap;
import java.io.*;
import java.util.*;
import java.util.regex.*;
public class SMAPSourceDebugExtension {
    public static void enhanceStackTrace ( Throwable t, ClassLoader cl, boolean keepOriginalFrames, String... packageNames ) throws IOException {
        enhanceStackTrace ( t, new HashMap<String, SMAPSourceDebugExtension>(), cl, keepOriginalFrames, packageNames );
    }
    public static void enhanceStackTrace ( Throwable t, Map<String, SMAPSourceDebugExtension> cache, ClassLoader cl, boolean keepOriginalFrames, String... packageNames ) throws IOException {
        StackTraceElement[] elements = t.getStackTrace();
        List<StackTraceElement> newElements = null;
        for ( int i = 0; i < elements.length; i++ ) {
            String className = elements[i].getClassName();
            SMAPSourceDebugExtension smap = cache.get ( className );
            if ( smap == null ) {
                boolean found = false;
                for ( String packageName : packageNames ) {
                    if ( className.startsWith ( packageName + "." ) ) {
                        found = true;
                        break;
                    }
                }
                if ( found || packageNames.length == 0 ) {
                    InputStream in = cl.getResourceAsStream ( className.replace ( '.', '/' ) + ".class" );
                    if ( in != null ) {
                        String value = extractSourceDebugExtension ( in );
                        in.close();
                        if ( value != null ) {
                            value = value.replaceAll ( "\r\n?", "\n" );
                            if ( value.startsWith ( "SMAP\n" ) ) {
                                smap = new SMAPSourceDebugExtension ( value );
                                cache.put ( className, smap );
                            }
                        }
                    }
                }
            }
            StackTraceElement newFrame = null;
            if ( smap != null ) {
                int[] inputLineInfo = smap.reverseLineMapping.get ( elements[i].getLineNumber() );
                if ( inputLineInfo != null && elements[i].getFileName().equals ( smap.generatedFileName ) ) {
                    FileInfo inputFileInfo = smap.fileinfo.get ( inputLineInfo[0] );
                    if ( inputFileInfo != null ) {
                        newFrame = new StackTraceElement ( "[" + smap.firstStratum + "]", inputFileInfo.path, inputFileInfo.name, inputLineInfo[1] );
                    }
                }
            }
            if ( newFrame != null ) {
                if ( newElements == null ) {
                    newElements = new ArrayList<StackTraceElement> ( Arrays.asList ( elements ).subList ( 0, i ) );
                }
                if ( keepOriginalFrames ) {
                    newElements.add ( elements[i] );
                }
                newElements.add ( newFrame );
            } else if ( newElements != null ) {
                newElements.add ( elements[i] );
            }
        }
        if ( newElements != null ) {
            t.setStackTrace ( newElements.toArray ( new StackTraceElement[newElements.size()] ) );
        }
        if ( t.getCause() != null ) {
            enhanceStackTrace ( t.getCause(), cache, cl, keepOriginalFrames, packageNames );
        }
    }
    private static String extractSourceDebugExtension ( InputStream in ) throws IOException {
        DataInputStream dis = new DataInputStream ( in );
        boolean[] isSourceDebugExtension;
        dis.skipBytes ( 8 );
        isSourceDebugExtension = new boolean[dis.readUnsignedShort()];
        int[] skipSizes = new int[] { 0, 0, 2, 4, 4, 0, 0, 2, 2, 4, 4, 4, 4, 2, 2, 3, 2, 2, 4 };
        for ( int i = 1; i < isSourceDebugExtension.length; i++ ) {
            byte type = dis.readByte();
            int skipSize;
            if ( type == 1 ) {
                String value = dis.readUTF();
                isSourceDebugExtension[i] = value.equals ( "SourceDebugExtension" );
                skipSize = 0;
            } else if ( type == 5 || type == 6 ) {
                skipSize = 8;
                i++;
            } else if ( type > 1 && type < 19 ) {
                skipSize = skipSizes[type];
            } else {
                skipSize = 2;
            }
            dis.skipBytes ( skipSize );
        }
        dis.skipBytes ( 6 );
        int ifaces = dis.readUnsignedShort();
        dis.skipBytes ( 2 * ifaces );
        for ( int k = 0; k < 2; k++ ) {
            int count = dis.readUnsignedShort();
            for ( int i = 0; i < count; i++ ) {
                dis.skipBytes ( 6 );
                int attrCount = dis.readUnsignedShort();
                for ( int j = 0; j < attrCount; j++ ) {
                    dis.skipBytes ( 2 );
                    int skip = dis.readInt();
                    dis.skipBytes ( skip );
                }
            }
        }
        int attrCount = dis.readUnsignedShort();
        for ( int i = 0; i < attrCount; i++ ) {
            int idx = dis.readUnsignedShort();
            int len = dis.readInt();
            if ( isSourceDebugExtension[idx] ) {
                byte[] buf = new byte[len];
                dis.readFully ( buf );
                return new String ( buf, "UTF-8" );
            } else {
                dis.skipBytes ( len );
            }
        }
        return null;
    }
    private final String generatedFileName, firstStratum;
    private final Map<Integer, FileInfo> fileinfo = new HashMap<Integer, FileInfo>();
    private final Map<Integer, int[]> reverseLineMapping = new HashMap<Integer, int[]>();
    private static final Pattern LINE_INFO_PATTERN = Pattern.compile ( "([0-9]+)(?:#([0-9]+))?(?:,([0-9]+))?:([0-9]+)(?:,([0-9]+))?" );
    private SMAPSourceDebugExtension ( String value ) {
        String[] lines = value.split ( "\n" );
        if ( !lines[0].equals ( "SMAP" ) || !lines[3].startsWith ( "*S " ) || !lines[4].equals ( "*F" ) ) {
            throw new IllegalArgumentException ( value );
        }
        generatedFileName = lines[1];
        firstStratum = lines[3].substring ( 3 );
        int idx = 5;
        while ( !lines[idx].startsWith ( "*" ) ) {
            String infoline = lines[idx++], path = null;
            if ( infoline.startsWith ( "+ " ) ) {
                path = lines[idx++];
                infoline = infoline.substring ( 2 );
            }
            int pos = infoline.indexOf ( " " );
            int filenum = Integer.parseInt ( infoline.substring ( 0, pos ) );
            String name = infoline.substring ( pos + 1 );
            fileinfo.put ( filenum, new FileInfo ( name, path == null ? name : path ) );
        }
        if ( lines[idx].equals ( "*L" ) ) {
            idx++;
            int lastLFI = 0;
            while ( !lines[idx].startsWith ( "*" ) ) {
                Matcher m = LINE_INFO_PATTERN.matcher ( lines[idx++] );
                if ( !m.matches() ) {
                    throw new IllegalArgumentException ( lines[idx - 1] );
                }
                int inputStartLine = Integer.parseInt ( m.group ( 1 ) );
                int lineFileID = m.group ( 2 ) == null ? lastLFI : Integer.parseInt ( m.group ( 2 ) );
                int repeatCount = m.group ( 3 ) == null ? 1 : Integer.parseInt ( m.group ( 3 ) );
                int outputStartLine = Integer.parseInt ( m.group ( 4 ) );
                int outputLineIncrement = m.group ( 5 ) == null ? 1 : Integer.parseInt ( m.group ( 5 ) );
                for ( int i = 0; i < repeatCount; i++ ) {
                    int[] inputMapping = new int[] { lineFileID, inputStartLine + i };
                    int baseOL = outputStartLine + i * outputLineIncrement;
                    for ( int ol = baseOL; ol < baseOL + outputLineIncrement; ol++ ) {
                        if ( !reverseLineMapping.containsKey ( ol ) ) {
                            reverseLineMapping.put ( ol, inputMapping );
                        }
                    }
                }
                lastLFI = lineFileID;
            }
        }
    }
    private static class FileInfo {
        public final String name, path;
        public FileInfo ( String name, String path ) {
            this.name = name;
            this.path = path;
        }
    }
}
