public class CustomScheduledAnnotationBeanPostProcessor implements BeanPostProcessor, Ordered, EmbeddedValueResolverAware, ApplicationContextAware, ApplicationListener<ContextRefreshedEvent>, DisposableBean {
    private static final Logger LOG = LoggerFactory.getLogger ( CustomScheduledAnnotationBeanPostProcessor.class );
    public Object postProcessBeforeInitialization ( Object bean, String beanName ) throws BeansException {
        return bean;
    }
    public Object postProcessAfterInitialization ( final Object bean, String beanName ) throws BeansException {
        final Class<?> targetClass = AopUtils.getTargetClass ( bean );
        ReflectionUtils.doWithMethods ( targetClass, new MethodCallback() {
            public void doWith ( Method method ) throws IllegalArgumentException, IllegalAccessException {
                Scheduled oldScheduledAnnotation = AnnotationUtils.getAnnotation ( method, Scheduled.class );
                if ( oldScheduledAnnotation != null ) {
                    LOG.info ( "@Scheduled found at method {}", method.getName() );
                    Assert.isTrue ( void.class.equals ( method.getReturnType() ), "Only void-returning methods may be annotated with @Scheduled." );
                    Assert.isTrue ( method.getParameterTypes().length == 0, "Only no-arg methods may be annotated with @Scheduled." );
                    if ( AopUtils.isJdkDynamicProxy ( bean ) ) {
                        try {
                            method = bean.getClass().getMethod ( method.getName(), method.getParameterTypes() );
                        } catch ( SecurityException ex ) {
                            ReflectionUtils.handleReflectionException ( ex );
                        } catch ( NoSuchMethodException ex ) {
                            throw new IllegalStateException ( String.format (
                                                                  "@Scheduled method '%s' found on bean target class '%s', " +
                                                                  "but not found in any interface(s) for bean JDK proxy. Either " +
                                                                  "pull the method up to an interface or switch to subclass (CGLIB) " +
                                                                  "proxies by setting proxy-target-class/proxyTargetClass " +
                                                                  "attribute to 'true'", method.getName(), targetClass.getSimpleName() ) );
                        }
                    }
                    Runnable runnable = new ScheduledMethodRunnable ( bean, method );
                    boolean processedSchedule = false;
                    String errorMessage = "Exactly one of 'cron', 'fixedDelay', or 'fixedRate' is required.";
                    String cron = oldScheduledAnnotation.cron();
                    if ( !"".equals ( cron ) ) {
                        processedSchedule = true;
                        if ( embeddedValueResolver != null ) {
                            cron = embeddedValueResolver.resolveStringValue ( cron );
                        }
                        cronTasks.put ( runnable, cron );
                    }
                    long fixedDelay = oldScheduledAnnotation.fixedDelay();
                    if ( fixedDelay >= 0 ) {
                        Assert.isTrue ( !processedSchedule, errorMessage );
                        processedSchedule = true;
                        fixedDelayTasks.put ( runnable, fixedDelay );
                    }
                    long fixedRate = oldScheduledAnnotation.fixedRate();
                    if ( fixedRate >= 0 ) {
                        Assert.isTrue ( !processedSchedule, errorMessage );
                        processedSchedule = true;
                        fixedRateTasks.put ( runnable, fixedRate );
                    }
                    Assert.isTrue ( processedSchedule, errorMessage );
                }
                CustomScheduled newScheduledAnnotation = AnnotationUtils.getAnnotation ( method, CustomScheduled.class );
                if ( newScheduledAnnotation != null ) {
                    LOG.info ( "@CustomScheduled found at method {}", method.getName() );
                    Assert.isTrue ( void.class.equals ( method.getReturnType() ), "Only void-returning methods may be annotated with @CustomScheduled." );
                    Assert.isTrue ( method.getParameterTypes().length == 0, "Only no-arg methods may be annotated with @CustomScheduled." );
                    if ( AopUtils.isJdkDynamicProxy ( bean ) ) {
                        try {
                            method = bean.getClass().getMethod ( method.getName(), method.getParameterTypes() );
                        } catch ( SecurityException ex ) {
                            ReflectionUtils.handleReflectionException ( ex );
                        } catch ( NoSuchMethodException ex ) {
                            throw new IllegalStateException ( String.format ( "@CustomScheduled method '%s' found on bean target class '%s', "
                                                              + "but not found in any interface(s) for bean JDK proxy. Either "
                                                              + "pull the method up to an interface or switch to subclass (CGLIB) "
                                                              + "proxies by setting proxy-target-class/proxyTargetClass " + "attribute to 'true'", method.getName(),
                                                              targetClass.getSimpleName() ) );
                        }
                    }
                    Runnable runnable = new ScheduledMethodRunnable ( bean, method );
                    boolean processedSchedule = false;
                    String errorMessage = "Exactly one of 'cron', 'fixedDelay', or 'fixedRate' is required.";
                    boolean numberFormatException = false;
                    String numberFormatErrorMessage = "Delay value is not a number!";
                    String cron = newScheduledAnnotation.cron();
                    if ( !"".equals ( cron ) ) {
                        processedSchedule = true;
                        if ( embeddedValueResolver != null ) {
                            cron = embeddedValueResolver.resolveStringValue ( cron );
                        }
                        cronTasks.put ( runnable, cron );
                        LOG.info ( "Put cron in tasks map with value {}", cron );
                    }
                    Long fixedDelay = null;
                    String resolverDelayCandidate = newScheduledAnnotation.fixedDelay();
                    if ( !"".equals ( resolverDelayCandidate ) ) {
                        try {
                            if ( embeddedValueResolver != null ) {
                                resolverDelayCandidate = embeddedValueResolver.resolveStringValue ( resolverDelayCandidate );
                                fixedDelay = Long.valueOf ( resolverDelayCandidate );
                            } else {
                                fixedDelay = Long.valueOf ( newScheduledAnnotation.fixedDelay() );
                            }
                        } catch ( NumberFormatException e ) {
                            numberFormatException = true;
                        }
                    }
                    Assert.isTrue ( !numberFormatException, numberFormatErrorMessage );
                    if ( fixedDelay != null && fixedDelay >= 0 ) {
                        Assert.isTrue ( !processedSchedule, errorMessage );
                        processedSchedule = true;
                        fixedDelayTasks.put ( runnable, fixedDelay );
                        LOG.info ( "Put fixedDelay in tasks map with value {}", fixedDelay );
                    }
                    Long fixedRate = null;
                    String resolverRateCandidate = newScheduledAnnotation.fixedRate();
                    if ( !"".equals ( resolverRateCandidate ) ) {
                        try {
                            if ( embeddedValueResolver != null ) {
                                fixedRate = Long.valueOf ( embeddedValueResolver.resolveStringValue ( resolverRateCandidate ) );
                            } else {
                                fixedRate = Long.valueOf ( newScheduledAnnotation.fixedRate() );
                            }
                        } catch ( NumberFormatException e ) {
                            numberFormatException = true;
                        }
                    }
                    Assert.isTrue ( !numberFormatException, numberFormatErrorMessage );
                    if ( fixedRate != null && fixedRate >= 0 ) {
                        Assert.isTrue ( !processedSchedule, errorMessage );
                        processedSchedule = true;
                        fixedRateTasks.put ( runnable, fixedRate );
                        LOG.info ( "Put fixedRate in tasks map with value {}", fixedRate );
                    }
                    Assert.isTrue ( processedSchedule, errorMessage );
                }
            }
        } );
        return bean;
    }
}
