import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Random;
import java.util.function.IntPredicate;
public final class TestStrings {
    final int MAX_STRING_SIZE = 1024 * 256;
    final int TRIES_PER_STRING_SIZE = 1000;
    public static void main ( String[] args ) throws Exception {
        new TestStrings().run();
    }
    void run() throws Exception {
        final List<Integer> sizes = new ArrayList<>();
        for ( int n = 0; n <= MAX_STRING_SIZE; n = ( n == 0 ? 1 : n * 2 ) ) {
            sizes.add ( n );
        }
        final Random random = new Random();
        System.out.println ( "Rate in nanoseconds per character inspected." );
        System.out.printf ( "==== FORWARDS (tries per size: %s) ==== \n", TRIES_PER_STRING_SIZE );
        printHeadings ( TRIES_PER_STRING_SIZE, random );
        for ( int size : sizes ) {
            reportResults ( size, test ( size, TRIES_PER_STRING_SIZE, random ) );
        }
        Collections.reverse ( sizes );
        System.out.println ( "" );
        System.out.println ( "Rate in nanoseconds per character inspected." );
        System.out.printf ( "==== BACKWARDS (tries per size: %s) ==== \n", TRIES_PER_STRING_SIZE );
        printHeadings ( TRIES_PER_STRING_SIZE, random );
        for ( int size : sizes ) {
            reportResults ( size, test ( size, TRIES_PER_STRING_SIZE, random ) );
        }
    }
    int charAtMethod1 ( final String data ) {
        final int len = data.length();
        for ( int i = 0; i < len; i++ ) {
            if ( data.charAt ( i ) <= ' ' ) {
                doThrow();
            }
        }
        return len;
    }
    int charAtMethod2 ( final String data ) {
        for ( int i = 0; i < data.length(); i++ ) {
            if ( data.charAt ( i ) <= ' ' ) {
                doThrow();
            }
        }
        return data.length();
    }
    int streamMethod ( final String data, final IntPredicate predicate ) {
        if ( data.chars().anyMatch ( predicate ) ) {
            doThrow();
        }
        return data.length();
    }
    int streamParallelMethod ( final String data, IntPredicate predicate ) {
        if ( data.chars().parallel().anyMatch ( predicate ) ) {
            doThrow();
        }
        return data.length();
    }
    int reuseBuffMethod ( final char[] reusable, final String data ) {
        final int len = data.length();
        data.getChars ( 0, len, reusable, 0 );
        for ( int i = 0; i < len; i++ ) {
            if ( reusable[i] <= ' ' ) {
                doThrow();
            }
        }
        return len;
    }
    int newMethod1 ( final String data ) {
        final int len = data.length();
        final char[] copy = data.toCharArray();
        for ( int i = 0; i < len; i++ ) {
            if ( copy[i] <= ' ' ) {
                doThrow();
            }
        }
        return len;
    }
    int newMethod2 ( final String data ) {
        for ( final char c : data.toCharArray() ) {
            if ( c <= ' ' ) {
                doThrow();
            }
        }
        return data.length();
    }
    int fieldMethod1 ( final Field field, final String data ) {
        try {
            final char[] chars = ( char[] ) field.get ( data );
            final int len = chars.length;
            for ( int i = 0; i < len; i++ ) {
                if ( chars[i] <= ' ' ) {
                    doThrow();
                }
            }
            return len;
        } catch ( Exception ex ) {
            throw new RuntimeException ( ex );
        }
    }
    int fieldMethod2 ( final Field field, final String data ) {
        final char[] chars;
        try {
            chars = ( char[] ) field.get ( data );
        } catch ( Exception ex ) {
            throw new RuntimeException ( ex );
        }
        for ( final char c : chars ) {
            if ( c <= ' ' ) {
                doThrow();
            }
        }
        return chars.length;
    }
    List<Jobber> makeTests ( String data ) throws Exception {
        final List<Jobber> tests = new ArrayList<Jobber>();
        tests.add ( new Jobber ( "charAt1" ) {
            int check() {
                return charAtMethod1 ( data );
            }
        } );
        tests.add ( new Jobber ( "charAt2" ) {
            int check() {
                return charAtMethod2 ( data );
            }
        } );
        tests.add ( new Jobber ( "stream" ) {
            final IntPredicate predicate = new IntPredicate() {
                public boolean test ( int value ) {
                    return value <= ' ';
                }
            };
            int check() {
                return streamMethod ( data, predicate );
            }
        } );
        tests.add ( new Jobber ( "streamPar" ) {
            final IntPredicate predicate = new IntPredicate() {
                public boolean test ( int value ) {
                    return value <= ' ';
                }
            };
            int check() {
                return streamParallelMethod ( data, predicate );
            }
        } );
        tests.add ( new Jobber ( "reuse" ) {
            final char[] cbuff = new char[MAX_STRING_SIZE];
            int check() {
                return reuseBuffMethod ( cbuff, data );
            }
        } );
        tests.add ( new Jobber ( "new1" ) {
            int check() {
                return newMethod1 ( data );
            }
        } );
        tests.add ( new Jobber ( "new2" ) {
            int check() {
                return newMethod2 ( data );
            }
        } );
        tests.add ( new Jobber ( "field1" ) {
            final Field field;
            {
                field = String.class.getDeclaredField ( "value" );
                field.setAccessible ( true );
            }
            int check() {
                return fieldMethod1 ( field, data );
            }
        } );
        tests.add ( new Jobber ( "field2" ) {
            final Field field;
            {
                field = String.class.getDeclaredField ( "value" );
                field.setAccessible ( true );
            }
            int check() {
                return fieldMethod2 ( field, data );
            }
        } );
        return tests;
    }
    abstract class Jobber {
        final String name;
        long nanos;
        long chars;
        long runs;
        Jobber ( String name ) {
            this.name = name;
        }
        abstract int check();
        final double nanosPerChar() {
            double charsPerRun = chars / runs;
            long nanosPerRun = nanos / runs;
            return charsPerRun == 0 ? nanosPerRun : nanosPerRun / charsPerRun;
        }
        final void run() {
            runs++;
            long time = System.nanoTime();
            chars += check();
            nanos += System.nanoTime() - time;
        }
    }
    private String makeTestString ( int testSize, char start, char end ) {
        Random r = new Random();
        char[] data = new char[testSize];
        for ( int i = 0; i < data.length; i++ ) {
            data[i] = ( char ) ( start + r.nextInt ( end ) );
        }
        return new String ( data );
    }
    public void doThrow() {
        throw new RuntimeException ( "Bzzzt -- Illegal Character!!" );
    }
    List<Jobber> test ( int size, int tries, Random random ) throws Exception {
        String data = makeTestString ( size, 'A', 'Z' );
        List<Jobber> tests = makeTests ( data );
        List<Jobber> copy = new ArrayList<> ( tests );
        while ( tries-- > 0 ) {
            Collections.shuffle ( copy, random );
            for ( Jobber ti : copy ) {
                ti.run();
            }
        }
        long runs = tests.get ( 0 ).runs;
        long count = tests.get ( 0 ).chars;
        for ( Jobber ti : tests ) {
            if ( ti.runs != runs && ti.chars != count ) {
                throw new Exception ( "Char counts should match if all correct algorithms" );
            }
        }
        return tests;
    }
    private void printHeadings ( final int TRIES_PER_STRING_SIZE, final Random random ) throws Exception {
        System.out.print ( "  Size" );
        for ( Jobber ti : test ( 0, TRIES_PER_STRING_SIZE, random ) ) {
            System.out.printf ( "%9s", ti.name );
        }
        System.out.println ( "" );
    }
    private void reportResults ( int size, List<Jobber> tests ) {
        System.out.printf ( "%6d", size );
        for ( Jobber ti : tests ) {
            System.out.printf ( "%,9.2f", ti.nanosPerChar() );
        }
        System.out.println ( "" );
    }
}
