package i3.util;
import java.util.AbstractSet;
import java.util.Collection;
import java.util.ConcurrentModificationException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.ListIterator;
import java.util.Map;
import java.util.NoSuchElementException;
public class LinkedSet<E> extends AbstractSet<E> {
    private Map<E, Node> m = new HashMap<E, Node>();
    protected Node head = new Node();
    private int monotonicallyIncreasing = 0;
    private int iteratorAddStep = 10;
    private int modCount;
    public ListIterator<E> from ( E elem ) {
        Node e = m.get ( elem );
        if ( e == null ) {
            throw new NoSuchElementException ( "the given element isn't part of the set" );
        }
        return new LinkedKeyIterator ( e );
    }
    @Override
    public ListIterator<E> iterator() {
        return new LinkedKeyIterator();
    }
    public boolean containsBefore ( E target, E limitElem ) {
        if ( isEmpty() ) {
            return false;
        }
        Integer targetN = m.get ( target ).relativeLocation;
        Integer highN = m.get ( limitElem ).relativeLocation;
        return targetN != null && highN != null && targetN < highN;
    }
    public boolean containsAfter ( E target, E previousElem ) {
        if ( isEmpty() ) {
            return false;
        }
        Integer targetN = m.get ( target ).relativeLocation;
        Integer low = m.get ( previousElem ).relativeLocation;
        return targetN != null && low != null && low < targetN;
    }
    @Override
    public boolean add ( E e ) {
        if ( !m.containsKey ( e ) ) {
            Node n = new Node ( e, monotonicallyIncreasing );
            monotonicallyIncreasing += iteratorAddStep;
            n.addBefore ( head );
            m.put ( e, n );
            return true;
        }
        return false;
    }
    @Override
    public int size() {
        return m.size();
    }
    @Override
    public boolean isEmpty() {
        return m.isEmpty();
    }
    @Override
    public boolean contains ( Object o ) {
        return m.containsKey ( o );
    }
    @Override
    public Object[] toArray() {
        Object[] result = new Object[size()];
        int i = 0;
        for ( E e : this ) {
            result[i++] = e;
        }
        return result;
    }
    @Override
    @SuppressWarnings ( "unchecked" )
    public <T> T[] toArray ( T[] a ) {
        int size = size();
        if ( a.length < size ) {
            a = ( T[] ) java.lang.reflect.Array.newInstance ( a.getClass().getComponentType(), size );
        }
        int i = 0;
        Object[] result = a;
        for ( E e : this ) {
            result[i++] = e;
        }
        if ( a.length > size ) {
            a[size] = null;
        }
        return a;
    }
    @Override
    public boolean remove ( Object o ) {
        Node n = m.remove ( o );
        if ( n != null ) {
            n.remove();
            return true;
        }
        return false;
    }
    @Override
    public boolean addAll ( Collection<? extends E> c ) {
        boolean changed = false;
        for ( E e : c ) {
            changed |= add ( e );
        }
        return changed;
    }
    @Override
    public boolean containsAll ( Collection<?> c ) {
        boolean all = true;
        for ( Object e : c ) {
            all &= m.containsKey ( e );
        }
        return all;
    }
    @Override
    public boolean retainAll ( Collection<?> c ) {
        boolean changed = false;
        Iterator<E> it = iterator();
        while ( it.hasNext() ) {
            E k = it.next();
            if ( !c.contains ( k ) ) {
                it.remove();
                changed = true;
            }
        }
        return changed;
    }
    @Override
    public void clear() {
        modCount++;
        head.after = head.before = head;
        m.clear();
    }
    @Override
    public String toString() {
        return  m.keySet().toString();
    }
    protected final class Node {
        Node before, after;
        int relativeLocation;
        E key;
        private void remove() {
            before.after = after;
            after.before = before;
            modCount++;
        }
        private void addBefore ( Node existingEntry ) {
            after = existingEntry;
            before = existingEntry.before;
            before.after = this;
            after.before = this;
            modCount++;
        }
        public Node() {
            after = before = this;
            relativeLocation = 0;
        }
        public Node ( E key, int value ) {
            this.key = key;
            this.relativeLocation = value;
        }
    }
    protected class LinkedKeyIterator implements ListIterator<E> {
        Node nextEntry;
        Node lastReturned;
        int expectedModCount = modCount;
        public LinkedKeyIterator() {
            nextEntry = head.after;
        }
        public LinkedKeyIterator ( Node startAt ) {
            nextEntry = startAt;
        }
        public boolean hasPrevious() {
            return nextEntry.before != head;
        }
        public boolean hasNext() {
            return nextEntry != head;
        }
        public E next() {
            if ( modCount != expectedModCount ) {
                throw new ConcurrentModificationException();
            }
            if ( nextEntry == head ) {
                throw new NoSuchElementException();
            }
            Node e = lastReturned = nextEntry;
            nextEntry = e.after;
            return e.key;
        }
        public E previous() {
            if ( modCount != expectedModCount ) {
                throw new ConcurrentModificationException();
            }
            if ( nextEntry.before == head ) {
                throw new NoSuchElementException();
            }
            Node e = lastReturned = nextEntry.before;
            nextEntry = e;
            return e.key;
        }
        public void remove() {
            if ( lastReturned == null ) {
                throw new IllegalStateException();
            }
            if ( modCount != expectedModCount ) {
                throw new ConcurrentModificationException();
            }
            m.remove ( lastReturned.key );
            nextEntry = lastReturned.after;
            lastReturned.remove();
            lastReturned = null;
            expectedModCount = modCount;
        }
        @Override
        public void set ( E e ) {
            if ( lastReturned == null ) {
                throw new IllegalStateException();
            }
            if ( modCount != expectedModCount ) {
                throw new ConcurrentModificationException();
            }
            if ( lastReturned.key.equals ( e ) ) {
                return;
            }
            m.remove ( lastReturned.key );
            lastReturned.key = e;
            Node previousKeyOwner = m.put ( e, lastReturned );
            if ( previousKeyOwner != null ) {
                if ( nextEntry == previousKeyOwner ) {
                    nextEntry = nextEntry.after;
                }
                previousKeyOwner.remove();
            }
            expectedModCount = ++modCount;
        }
        @Override
        public void add ( E e ) {
            if ( modCount != expectedModCount ) {
                throw new ConcurrentModificationException();
            }
            int candidateLoc = nextEntry.before.relativeLocation + 1;
            if ( candidateLoc == nextEntry.relativeLocation ) {
                iteratorAddStep *= 1.6;
                for ( Node current = nextEntry; current != head; current = current.after ) {
                    current.relativeLocation = current.relativeLocation + iteratorAddStep;
                }
            }
            Node n = m.get ( e );
            if ( n == null ) {
                n = new Node ( e, candidateLoc );
                m.put ( e, n );
            } else {
                n.relativeLocation = candidateLoc;
                if ( nextEntry == n ) {
                    nextEntry = nextEntry.after;
                }
                n.remove();
            }
            n.addBefore ( nextEntry );
            expectedModCount = modCount;
        }
        @Override
        public int nextIndex() {
            throw new UnsupportedOperationException ( "Not supported yet." );
        }
        @Override
        public int previousIndex() {
            throw new UnsupportedOperationException ( "Not supported yet." );
        }
    }
}
