package com.test;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
interface IProducer<T extends Comparable<T>> {
    public Map<Integer, T> getResults();
}
class ProducerA implements IProducer<String> {
    @Override
    public Map<Integer, String> getResults() {
        Map<Integer, String> result = new HashMap<Integer, String>();
        result.put ( 1, "A" );
        result.put ( 2, "B" );
        result.put ( 3, "B" );
        return result;
    }
}
class ProducerB implements IProducer<Integer> {
    @Override
    public Map<Integer, Integer> getResults() {
        Map<Integer, Integer> result = new HashMap<Integer, Integer>();
        result.put ( 1, 10 );
        result.put ( 2, 30 );
        result.put ( 3, 30 );
        return result;
    }
}
interface IEvaluator {
    public <T extends Comparable<T>> double evaluate ( Map<Integer, T> results,
            Map<Integer, Double> groundTruth );
}
class KendallTauB implements IEvaluator {
    @Override
    public <T extends Comparable<T>> double evaluate ( Map<Integer, T> results,
            Map<Integer, Double> groundTruth ) {
        int concordant = 0, discordant = 0, tiedRanks = 0, tiedCapabilities = 0;
        for ( Entry<Integer, T> rank1 : results.entrySet() ) {
            for ( Entry<Integer, T> rank2 : results.entrySet() ) {
                if ( rank1.getKey() < rank2.getKey() ) {
                    final T r1 = rank1.getValue();
                    final T r2 = rank2.getValue();
                    final Double c1 = groundTruth.get ( rank1.getKey() );
                    final Double c2 = groundTruth.get ( rank2.getKey() );
                    final int ranksDiff = r1.compareTo ( r2 );
                    final int actualDiff = c1.compareTo ( c2 );
                    if ( ranksDiff * actualDiff > 0 ) {
                        concordant++;
                    } else if ( ranksDiff * actualDiff < 0 ) {
                        discordant++;
                    } else {
                        if ( ranksDiff == 0 ) {
                            tiedRanks++;
                        }
                        if ( actualDiff == 0 ) {
                            tiedCapabilities++;
                        }
                    }
                }
            }
        }
        final double n = results.size() * ( results.size() - 1d ) / 2d;
        return ( concordant - discordant )
               / Math.sqrt ( ( n - tiedRanks ) * ( n - tiedCapabilities ) );
    }
}
public class Simulator {
    public static void main ( String[] args ) {
        Map<Integer, Double> groundTruth = new HashMap<Integer, Double>();
        groundTruth.put ( 1, 1d );
        groundTruth.put ( 2, 2d );
        groundTruth.put ( 3, 3d );
        List<IProducer<?>> producerImplementations = lookUpProducers();
        List<IEvaluator> evaluatorImplementations = lookUpEvaluators();
        IProducer<?> producer = producerImplementations.get ( 0 );
        IEvaluator evaluator = evaluatorImplementations.get ( 0 );
        double score = evaluator.evaluate ( producer.getResults(), groundTruth );
        System.out.printf ( "Score is %.2f\n", score );
    }
    public static List<IProducer<?>> lookUpProducers() {
        List<IProducer<?>> producers = new ArrayList<IProducer<?>>();
        producers.add ( new ProducerA() );
        producers.add ( new ProducerB() );
        return producers;
    }
    public static List<IEvaluator> lookUpEvaluators() {
        List<IEvaluator> evaluators = new ArrayList<IEvaluator>();
        evaluators.add ( new KendallTauB() );
        return evaluators;
    }
}
