package dirwatch;
import java.nio.file.*;
import static java.nio.file.StandardWatchEventKinds.ENTRY_CREATE;
import static java.nio.file.StandardWatchEventKinds.OVERFLOW;
import static java.nio.file.LinkOption.*;
import java.nio.file.attribute.*;
import java.io.*;
import java.util.*;
import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.TimeUnit;
public class WatchDir {
    private final WatchService watcher;
    private final Map<WatchKey, Path> keys;
    private final boolean recursive;
    private boolean trace = false;
    private BlockingQueue<String> fileProcessingQueue;
    public final BlockingQueue<String> processedFileQueue;
    private volatile boolean closeProcessingThread;
    private volatile boolean closeWatcherThread;
    private void processFiles() {
        System.out.println ( "DirWatchProcessingThread Started" );
        String fileName;
        outerLoop: while ( !closeProcessingThread || !fileProcessingQueue.isEmpty() ) {
            try {
                fileName = fileProcessingQueue.poll ( 1000, TimeUnit.MILLISECONDS );
            } catch ( InterruptedException ie ) {
                fileName = null;
            }
            if ( fileName == null || fileName.equals ( "" ) ) {
                continue outerLoop;
            }
            long startTime = System.currentTimeMillis();
            innerLoop: while ( true ) {
                FileInputStream fis = null;
                File file = new File ( fileName );
                try {
                    fis = new FileInputStream ( fileName );
                    break innerLoop;
                } catch ( FileNotFoundException fnfe ) {
                    if ( !file.exists() || file.isDirectory() ) {
                        System.out.println ( "File: '" + fileName + "has been deleted in file system or it is not file. Not processing this file." );
                        continue outerLoop;
                    }
                    try {
                        Thread.sleep ( WatchDirParameters.millisToPuaseForFileLock );
                    } catch ( InterruptedException ie ) {
                    }
                    if ( ( System.currentTimeMillis() - startTime ) > WatchDirParameters.millisToSwapFileForUnlocking ) {
                        if ( fileProcessingQueue.offer ( fileName ) ) {
                            continue outerLoop;
                        } else {
                            startTime = System.currentTimeMillis();
                            continue innerLoop;
                        }
                    }
                } finally {
                    if ( fis != null ) {
                        try {
                            fis.close();
                        } catch ( IOException ioe ) {
                            ioe.printStackTrace();
                        }
                    }
                }
            }
            System.out.println ( "Queuing File: " + fileName );
            processedLoop: while ( true ) {
                try {
                    if ( processedFileQueue.offer ( fileName, 1000, TimeUnit.MILLISECONDS ) ) {
                        break processedLoop;
                    }
                } catch ( InterruptedException ie ) {
                }
            }
        }
        closeWatcherThread = true;
        closeProcessingThread = true;
        System.out.println ( "DirWatchProcessingThread Exited" );
    }
    private void processEvents() {
        System.out.println ( "DirWatcherThread started." );
        while ( !closeWatcherThread ) {
            WatchKey key;
            try {
                key = watcher.take();
            } catch ( InterruptedException x ) {
                continue;
            } catch ( ClosedWatchServiceException cwse ) {
                break;
            }
            Path dir = keys.get ( key );
            if ( dir == null ) {
                System.err.println ( "WatchKey not recognized!!" );
                continue;
            }
            try {
                for ( WatchEvent<?> event : key.pollEvents() ) {
                    WatchEvent.Kind kind = event.kind();
                    if ( kind == OVERFLOW ) {
                        continue;
                    }
                    WatchEvent<Path> ev = cast ( event );
                    Path name = ev.context();
                    Path child = dir.resolve ( name );
                    if ( kind.equals ( ENTRY_CREATE ) ) {
                        if ( recursive ) {
                            try {
                                if ( Files.isDirectory ( child, NOFOLLOW_LINKS ) ) {
                                    registerAll ( child );
                                    continue;
                                }
                            } catch ( IOException x ) {
                            }
                        }
                        while ( true ) {
                            if ( fileProcessingQueue.remainingCapacity() < 2 ) {
                                try {
                                    Thread.sleep ( 200 );
                                } catch ( InterruptedException ie ) {
                                }
                                continue;
                            }
                            if ( !fileProcessingQueue.offer ( child.toString() ) ) {
                                continue;
                            } else {
                                break;
                            }
                        }
                    }
                }
                boolean valid = key.reset();
                if ( !valid ) {
                    keys.remove ( key );
                    if ( keys.isEmpty() ) {
                        break;
                    }
                }
            } catch ( ClosedWatchServiceException cwse ) {
                break;
            }
        }
        closeProcessingThread = true;
        closeWatcherThread = true;
        System.out.println ( "DirWatcherThread exited." );
    }
    public void stopWatching() {
        try {
            watcher.close();
        } catch ( IOException ioe ) {
        }
        closeProcessingThread = true;
        closeWatcherThread = true;
    }
    public static WatchDir watchDirectory ( String dirName, boolean recursive ) throws InvalidPathException, IOException, Exception {
        try {
            Path dir = Paths.get ( dirName );
            final WatchDir watchDir = new WatchDir ( dir, recursive );
            watchDir.closeProcessingThread = false;
            watchDir.closeWatcherThread = false;
            new Thread ( new Runnable() {
                public void run() {
                    watchDir.processFiles();
                }
            }, "DirWatchProcessingThread" ).start();
            new Thread ( new Runnable() {
                public void run() {
                    watchDir.processEvents();
                }
            }, "DirWatcherThread" ).start();
            return watchDir;
        } catch ( InvalidPathException ipe ) {
            throw ipe;
        } catch ( IOException ioe ) {
            throw ioe;
        } catch ( Exception e ) {
            throw e;
        }
    }
    @SuppressWarnings ( "unchecked" )
    private static <T> WatchEvent<T> cast ( WatchEvent<?> event ) {
        return ( WatchEvent<T> ) event;
    }
    private void register ( Path dir ) throws IOException {
        WatchKey key = dir.register ( watcher, ENTRY_CREATE );
        if ( trace ) {
            Path prev = keys.get ( key );
            if ( prev == null ) {
                System.out.format ( "register: %s\n", dir );
            } else {
                if ( !dir.equals ( prev ) ) {
                    System.out.format ( "update: %s -> %s\n", prev, dir );
                }
            }
        }
        keys.put ( key, dir );
    }
    private void registerAll ( final Path start ) throws IOException {
        Files.walkFileTree ( start, new SimpleFileVisitor<Path>() {
            @Override
            public FileVisitResult preVisitDirectory ( Path dir, BasicFileAttributes attrs ) throws IOException {
                register ( dir );
                return FileVisitResult.CONTINUE;
            }
        } );
    }
    private WatchDir ( Path dir, boolean recursive ) throws IOException {
        fileProcessingQueue = new ArrayBlockingQueue<String> ( WatchDirParameters.fileQueueSize, false );
        processedFileQueue = new ArrayBlockingQueue<String> ( WatchDirParameters.fileQueueSize, false );
        this.watcher = FileSystems.getDefault().newWatchService();
        this.keys = new HashMap<WatchKey, Path>();
        this.recursive = recursive;
        if ( recursive ) {
            System.out.format ( "Scanning %s ...\n", dir );
            registerAll ( dir );
            System.out.println ( "Done." );
        } else {
            register ( dir );
        }
        this.trace = true;
    }
}
