public class EntityArgumentResolver implements HandlerMethodArgumentResolver {
    @Autowired
    private SessionFactory sessionFactory;
    private final ObjectMapper objectMapper = new ObjectMapper();
    private static final Logger log = Logger.getLogger ( EntityArgumentResolver.class );
    private boolean doLog = false;
    @Override
    public boolean supportsParameter ( MethodParameter parameter ) {
        return parameter.getParameterType().getAnnotation ( Entity.class ) != null;
    }
    @Override
    public Object resolveArgument ( MethodParameter parameter, ModelAndViewContainer mavContainer, NativeWebRequest webRequest, WebDataBinderFactory binderFactory ) throws Exception {
        HttpServletRequest request = webRequest.getNativeRequest ( HttpServletRequest.class );
        String requestBody = IOUtils.toString ( request.getReader() );
        Class<?> targetClass = parameter.getParameterType();
        Object entity = this.parse ( requestBody, targetClass );
        Object entityId = getId ( entity );
        if ( doLog ) {
            log.info ( requestBody );
        }
        if ( entityId != null ) {
            return copyObjectToPersistedEntity ( entity, getKeyValueMap ( requestBody ), entityId );
        } else {
            return entity;
        }
    }
    @SuppressWarnings ( "unchecked" )
    private Map<String, Object> getKeyValueMap ( String rawJson ) throws JsonParseException, JsonMappingException, IOException {
        return objectMapper.readValue ( rawJson, HashMap.class );
    }
    private Object copyObjectToPersistedEntity ( Object changesObject, Map<String, Object> changesMap, Object id ) throws SecurityException, NoSuchMethodException, IllegalArgumentException, IllegalAccessException, InvocationTargetException {
        Session session = sessionFactory.openSession();
        Object persistedObject =
            session.get ( changesObject.getClass(), ( Serializable ) id );
        session.close();
        if ( persistedObject == null ) {
            throw new ValidationException ( changesObject.getClass().getSimpleName() + " #" + id + " not found." );
        }
        Class<?> clazz = persistedObject.getClass();
        for ( Method getterMethod : ReflectionUtils.getAllDeclaredMethods ( clazz ) ) {
            Column column = getterMethod.getAnnotation ( Column.class );
            if ( column == null ) {
                continue;
            }
            if ( !column.updatable() ) {
                continue;
            }
            if ( !changesMap.containsKey ( BeanUtils.toFieldName ( getterMethod ) ) ) {
                continue;
            }
            if ( ObjectUtils.equals ( getterMethod.invoke ( persistedObject ), getterMethod.invoke ( changesObject ) ) ) {
                continue;
            }
            log.info ( "Update " + clazz.getSimpleName() + "(" + id + ") [" + column.name() + "]" );
            Object obj = getterMethod.invoke ( changesObject );
            Method setter = BeanUtils.toSetter ( getterMethod );
            setter.invoke ( persistedObject, obj );
        }
        return persistedObject;
    }
    private Object getId ( Object entity ) throws IllegalArgumentException, IllegalAccessException, InvocationTargetException {
        for ( Method method : ReflectionUtils.getAllDeclaredMethods ( entity.getClass() ) ) {
            if ( method.getAnnotation ( Id.class ) != null ) {
                method.setAccessible ( true );
                return method.invoke ( entity );
            }
        }
        return null;
    }
    private <T> T parse ( String json, Class<T> clazz ) throws JsonParseException, IOException {
        try {
            return objectMapper.readValue ( json, clazz );
        } catch ( JsonMappingException e ) {
            throw new ValidationException ( e );
        }
    }
    public void setDoLog ( boolean doLog ) {
        this.doLog = doLog;
    }
}
