public class Message {
    private static final int CLIENT_PORT = 8000;
    private final Client[] to;
    private final String contents;
    private final StatusListener listener;
    private final List<Socket> clientConnections;
    private final BlockingQueue<Status> statusQueue;
    public Message ( Client[] to, String contents, StatusListener listener ) {
        this.to = to;
        this.contents = contents;
        this.listener = listener;
        clientConnections = Collections.synchronizedList ( new ArrayList<Socket>() );
        statusQueue = new ArrayBlockingQueue<Status> ( to.length * 3 );
    }
    public void dispatch() {
        sendContentsToEachClientAsynchronously();
        Status statusChangeReceived;
        do {
            try {
                statusChangeReceived = statusQueue.take();
            } catch ( InterruptedException ignore ) {
                break;
            }
        } while ( listener.updateStatus ( statusChangeReceived ) );
        closeRemainingClientConnections();
    }
    private void closeRemainingClientConnections() {
        for ( Socket connection : clientConnections ) {
            try {
                connection.close();
            } catch ( IOException ignore ) {
            }
        }
        clientConnections.clear();
    }
    private void sendContentsToEachClientAsynchronously() {
        for ( Client client : to ) {
            System.out.println ( "Started new thread" );
            new Thread ( new MessageDispatcher ( client ) ).start();
        }
    }
    private final class MessageDispatcher implements Runnable {
        private final Client client;
        MessageDispatcher ( Client client ) {
            this.client = client;
        }
        public void run() {
            try {
                communicateWithClient();
            } catch ( IOException e ) {
                throw new RuntimeException ( e );
            }
        }
        private void communicateWithClient() throws IOException {
            Socket connection = new Socket ( client.getAddress(), CLIENT_PORT );
            try {
                clientConnections.add ( connection );
                sendMessage ( connection.getOutputStream() );
                readRequiredReceipts ( connection.getInputStream() );
            } finally {
                connection.close();
            }
        }
        private void sendMessage ( OutputStream output ) {
            PrintWriter out = new PrintWriter ( output, true );
            out.println ( contents );
            statusQueue.add ( new Status ( client, "DISPATCHED" ) );
        }
        private void readRequiredReceipts ( InputStream input ) throws IOException {
            BufferedReader in = new BufferedReader ( new InputStreamReader ( input ) );
            in.readLine();
            statusQueue.add ( new Status ( client, "DISPLAYED" ) );
            in.readLine();
            statusQueue.add ( new Status ( client, "READ" ) );
        }
    }
}
