import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import org.apache.pdfbox.pdmodel.PDDocument;
import org.apache.pdfbox.pdmodel.PDPage;
import org.apache.pdfbox.pdmodel.PDResources;
import org.apache.pdfbox.pdmodel.encryption.AccessPermission;
import org.apache.pdfbox.pdmodel.encryption.StandardDecryptionMaterial;
import org.apache.pdfbox.pdmodel.graphics.xobject.PDXObject;
import org.apache.pdfbox.pdmodel.graphics.xobject.PDXObjectForm;
import org.apache.pdfbox.pdmodel.graphics.xobject.PDXObjectImage;
public class ExtractImages {
    private int imageCounter = 1;
    private static final String PASSWORD = "-password";
    private static final String PREFIX = "-prefix";
    private static final String ADDKEY = "-addkey";
    private static final String NONSEQ = "-nonSeq";
    private ExtractImages() {
    }
    public static void main ( String[] args ) throws Exception {
        ExtractImages extractor = new ExtractImages();
        extractor.extractImages ( args );
    }
    private void extractImages ( String[] args ) throws Exception {
        if ( args.length < 1 || args.length > 4 ) {
            usage();
        } else {
            String pdfFile = null;
            String password = "";
            String prefix = null;
            boolean addKey = false;
            boolean useNonSeqParser = false;
            for ( int i = 0; i < args.length; i++ ) {
                if ( args[i].equals ( PASSWORD ) ) {
                    i++;
                    if ( i >= args.length ) {
                        usage();
                    }
                    password = args[i];
                } else if ( args[i].equals ( PREFIX ) ) {
                    i++;
                    if ( i >= args.length ) {
                        usage();
                    }
                    prefix = args[i];
                } else if ( args[i].equals ( ADDKEY ) ) {
                    addKey = true;
                } else if ( args[i].equals ( NONSEQ ) ) {
                    useNonSeqParser = true;
                } else {
                    if ( pdfFile == null ) {
                        pdfFile = args[i];
                    }
                }
            }
            if ( pdfFile == null ) {
                usage();
            } else {
                if ( prefix == null && pdfFile.length() > 4 ) {
                    prefix = pdfFile.substring ( 0, pdfFile.length() - 4 );
                }
                PDDocument document = null;
                try {
                    if ( useNonSeqParser ) {
                        document = PDDocument.loadNonSeq ( new File ( pdfFile ), null, password );
                    } else {
                        document = PDDocument.load ( pdfFile );
                        if ( document.isEncrypted() ) {
                            StandardDecryptionMaterial spm = new StandardDecryptionMaterial ( password );
                            document.openProtection ( spm );
                        }
                    }
                    AccessPermission ap = document.getCurrentAccessPermission();
                    if ( ! ap.canExtractContent() ) {
                        throw new IOException (
                            "Error: You do not have permission to extract images." );
                    }
                    List pages = document.getDocumentCatalog().getAllPages();
                    Iterator iter = pages.iterator();
                    while ( iter.hasNext() ) {
                        PDPage page = ( PDPage ) iter.next();
                        PDResources resources = page.getResources();
                        processResources ( resources, prefix, addKey );
                    }
                } finally {
                    if ( document != null ) {
                        document.close();
                    }
                }
            }
        }
    }
    private void processResources ( PDResources resources, String prefix, boolean addKey ) throws IOException {
        if ( resources == null ) {
            return;
        }
        Map<String, PDXObject> xobjects = resources.getXObjects();
        if ( xobjects != null ) {
            Iterator<String> xobjectIter = xobjects.keySet().iterator();
            while ( xobjectIter.hasNext() ) {
                String key = xobjectIter.next();
                PDXObject xobject = xobjects.get ( key );
                if ( xobject instanceof PDXObjectImage ) {
                    PDXObjectImage image = ( PDXObjectImage ) xobject;
                    String name = null;
                    if ( addKey ) {
                        name = getUniqueFileName ( prefix + "_" + key, image.getSuffix() );
                    } else {
                        name = getUniqueFileName ( prefix, image.getSuffix() );
                    }
                    System.out.println ( "Writing image:" + name );
                    image.write2file ( name );
                } else if ( xobject instanceof PDXObjectForm ) {
                    PDXObjectForm xObjectForm = ( PDXObjectForm ) xobject;
                    PDResources formResources = xObjectForm.getResources();
                    processResources ( formResources, prefix, addKey );
                }
            }
        }
    }
    private String getUniqueFileName ( String prefix, String suffix ) {
        String uniqueName = null;
        File f = null;
        while ( f == null || f.exists() ) {
            uniqueName = prefix + "-" + imageCounter;
            f = new File ( uniqueName + "." + suffix );
            imageCounter++;
        }
        return uniqueName;
    }
    private static void usage() {
        System.err.println ( "Usage: java org.apache.pdfbox.ExtractImages [OPTIONS] <PDF file>\n" +
                             "  -password  <password>        Password to decrypt document\n" +
                             "  -prefix  <image-prefix>      Image prefix(default to pdf name)\n" +
                             "  -addkey                      add the internal image key to the file name\n" +
                             "  -nonSeq                      Enables the new non-sequential parser\n" +
                             "  <PDF file>                   The PDF document to use\n"
                           );
        System.exit ( 1 );
    }
}
