package advertising;
import java.util.Scanner;
import java.math.BigDecimal;
enum Graphic {
    NONE ( 0 ),
    SMALL ( 1907 ),
    MEDIUM ( 2476 ),
    LARGE ( 2933 );
    private final int cost;
    private Graphic ( int cost ) {
        this.cost = cost;
    }
    public int getCost() {
        return cost;
    }
}
enum LinkPlan {
    NONE ( 0 ),
    PREPAID ( 1495 ),
    POSTPAID ( 1495 * 3 - ( 1495 * 3 ) / 10 );
    private final int cost;
    private LinkPlan ( int cost ) {
        this.cost = cost;
    }
    public int getCost() {
        return cost;
    }
}
class Advertisement {
    private final int wordCount;
    private final LinkPlan linkPlan;
    private final Graphic graphic;
    public Advertisement ( int wordCount, LinkPlan linkPlan, Graphic graphic ) {
        this.wordCount = wordCount;
        this.linkPlan = linkPlan;
        this.graphic = graphic;
    }
    public int getWordCost() {
        if ( wordCount > 35 ) {
            return 32 * wordCount;
        }
        return 300 + Math.min ( wordCount, 25 ) * 40
               + Math.min ( wordCount - 25, 0 ) * 35;
    }
    public void printAllCosts() {
        System.out.printf ( "\t\t%-16s %11s\n", "Category", "Cost" );
        printCost ( "Text", getWordCost() );
        printCost ( "Link", linkPlan.getCost() );
        printCost ( "Graphic", graphic.getCost() );
        int total = getWordCost() + linkPlan.getCost() + graphic.getCost();
        printCost ( "Total", total );
        int gst = total / 20;
        printCost ( "GST", gst );
        printCost ( "Total with GST", total + gst );
    }
    private void printCost ( String category, int cents ) {
        BigDecimal dollars = new BigDecimal ( cents ).scaleByPowerOfTen ( -2 );
        System.out.printf ( "\t\t%-16s %11.2f\n", category, dollars );
    }
}
public class EntryPoint {
    public static void main ( String[] args ) {
        Scanner scanner = new Scanner ( System.in );
        System.out.println ( "Welcome!" );
        int wordCount = readWordCount ( scanner );
        LinkPlan linkPlan = readLinkPlan ( scanner );
        Graphic graphic = readGraphic ( scanner );
        Advertisement advert = new Advertisement ( wordCount, linkPlan, graphic );
        advert.printAllCosts();
    }
    private static int readWordCount ( Scanner scanner ) {
        System.out.print ( "Enter the number of words in your ad: " );
        return scanner.nextInt();
    }
    private static LinkPlan readLinkPlan ( Scanner scanner ) {
        System.out.print ( "Would you like to add a link (y = yes or n = no)? " );
        char addLink = readSingleCharacter ( scanner, "yn" );
        LinkPlan linkPlan;
        if ( addLink == 'n' ) {
            return LinkPlan.NONE;
        }
        System.out.print ( "Would you like to pay 3 months in advance " +
                           "(y = yes or n = no)? " );
        char advancePay = readSingleCharacter ( scanner, "yn" );
        return advancePay == 'y' ? LinkPlan.PREPAID : LinkPlan.POSTPAID;
    }
    private static Graphic readGraphic ( Scanner scanner ) {
        System.out.print ( "Would you like to add graphics/pictures? " +
                           "(s = small, m = medium, l = large or n = None)? " );
        char graphic = readSingleCharacter ( scanner, "smln" );
        switch ( graphic ) {
        case 's':
            return Graphic.SMALL;
        case 'm':
            return Graphic.MEDIUM;
        case 'l':
            return Graphic.LARGE;
        case 'n':
            return Graphic.NONE;
        default:
            throw new IllegalStateException ( "Unexpected state; graphic=" +
                                              graphic );
        }
    }
    private static char readSingleCharacter ( Scanner scanner,
            String validOptions ) {
        while ( true ) {
            String input = scanner.next();
            if ( input.length() != 1 || !validOptions.contains ( input ) ) {
                System.out.print ( "Invalid value. Please try again: " );
                continue;
            }
            return input.charAt ( 0 );
        }
    }
}
