public final class ValidationFilterFactory implements ResourceFilterFactory {
    @Override
    public List<ResourceFilter> create ( AbstractMethod abstractMethod ) {
        final ImmutableSet.Builder<String> requiredQueryParamsBuilder =
            ImmutableSet.builder();
        final ImmutableList<Parameter> params =
            Invokable.from ( abstractMethod.getMethod() ).getParameters();
        for ( Parameter param : params ) {
            if ( !param.isAnnotationPresent ( Nullable.class ) ) {
                @Nullable final QueryParam queryParam =
                    param.getAnnotation ( QueryParam.class );
                if ( queryParam != null ) {
                    requiredQueryParamsBuilder.add ( queryParam.value() );
                }
            }
        }
        return Collections.<ResourceFilter>singletonList (
                   new ValidationFilter ( requiredQueryParamsBuilder.build() )
               );
    }
    private static final class ValidationFilter implements ResourceFilter {
        final ImmutableSet<String> requiredQueryParams;
        private ValidationFilter ( ImmutableSet<String> requiredQueryParams ) {
            this.requiredQueryParams = requiredQueryParams;
        }
        @Override
        public ContainerRequestFilter getRequestFilter() {
            return new ContainerRequestFilter() {
                @Override
                public ContainerRequest filter ( ContainerRequest request ) {
                    final Collection<String> missingRequiredParams =
                        Sets.difference (
                            requiredQueryParams,
                            request.getQueryParameters().keySet()
                        );
                    if ( !missingRequiredParams.isEmpty() ) {
                        final String message =
                            "Required query params missing: " +
                            Joiner.on ( ", " ).join ( missingRequiredParams );
                        final Response response = Response
                                                  .status ( Status.BAD_REQUEST )
                                                  .entity ( message )
                                                  .build();
                        throw new WebApplicationException ( response );
                    }
                    return request;
                }
            };
        }
        @Override
        public ContainerResponseFilter getResponseFilter() {
            return null;
        }
    }
}
