package org.eclipse.jface.viewers;
import java.util.ArrayList;
import java.util.List;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.graphics.TextStyle;
import org.eclipse.jface.preference.JFacePreferences;
import org.eclipse.jface.resource.ColorRegistry;
import org.eclipse.jface.resource.JFaceResources;
public class StyledString {
    public static abstract class Styler {
        public abstract void applyStyles ( TextStyle textStyle );
    }
    public static final Styler QUALIFIER_STYLER = createColorRegistryStyler (
                JFacePreferences.QUALIFIER_COLOR, null );
    public static final Styler COUNTER_STYLER = createColorRegistryStyler (
                JFacePreferences.COUNTER_COLOR, null );
    public static final Styler DECORATIONS_STYLER = createColorRegistryStyler (
                JFacePreferences.DECORATIONS_COLOR, null );
    public static Styler createColorRegistryStyler ( String foregroundColorName,
            String backgroundColorName ) {
        return new DefaultStyler ( foregroundColorName, backgroundColorName );
    }
    private static final StyleRange[] EMPTY = new StyleRange[0];
    private StringBuffer fBuffer;
    private StyleRunList fStyleRuns;
    public StyledString() {
        fBuffer = new StringBuffer();
        fStyleRuns = null;
    }
    public StyledString ( String string ) {
        this ( string, null );
    }
    public StyledString ( String string, Styler styler ) {
        this();
        append ( string, styler );
    }
    public String getString() {
        return fBuffer.toString();
    }
    public String toString() {
        return getString();
    }
    public int length() {
        return fBuffer.length();
    }
    public StyledString append ( String string ) {
        return append ( string, null );
    }
    public StyledString append ( char[] chars ) {
        return append ( chars, null );
    }
    public StyledString append ( char ch ) {
        return append ( String.valueOf ( ch ), null );
    }
    public StyledString append ( StyledString string ) {
        if ( string.length() == 0 ) {
            return this;
        }
        int offset = fBuffer.length();
        fBuffer.append ( string.toString() );
        List otherRuns = string.fStyleRuns;
        if ( otherRuns != null && !otherRuns.isEmpty() ) {
            for ( int i = 0; i < otherRuns.size(); i++ ) {
                StyleRun curr = ( StyleRun ) otherRuns.get ( i );
                if ( i == 0 && curr.offset != 0 ) {
                    appendStyleRun ( null, offset );
                }
                appendStyleRun ( curr.style, offset + curr.offset );
            }
        } else {
            appendStyleRun ( null, offset );
        }
        return this;
    }
    public StyledString append ( char ch, Styler styler ) {
        return append ( String.valueOf ( ch ), styler );
    }
    public StyledString append ( String string, Styler styler ) {
        if ( string.length() == 0 ) {
            return this;
        }
        int offset = fBuffer.length();
        fBuffer.append ( string );
        appendStyleRun ( styler, offset );
        return this;
    }
    public StyledString append ( char[] chars, Styler styler ) {
        if ( chars.length == 0 ) {
            return this;
        }
        int offset = fBuffer.length();
        fBuffer.append ( chars );
        appendStyleRun ( styler, offset );
        return this;
    }
    public StyledString insert ( char ch, int offset ) throws StringIndexOutOfBoundsException {
        if ( offset < 0 || offset > fBuffer.length() ) {
            throw new StringIndexOutOfBoundsException (
                "Invalid offset (" + offset + ")" );
        }
        if ( hasRuns() ) {
            int runIndex = findRun ( offset );
            if ( runIndex < 0 ) {
                runIndex = -runIndex - 1;
            } else {
                runIndex = runIndex + 1;
            }
            StyleRunList styleRuns = getStyleRuns();
            final int size = styleRuns.size();
            for ( int i = runIndex; i < size; i++ ) {
                StyleRun run = styleRuns.getRun ( i );
                run.offset++;
            }
        }
        fBuffer.insert ( offset, ch );
        return this;
    }
    public void setStyle ( int offset, int length, Styler styler ) throws StringIndexOutOfBoundsException {
        if ( offset < 0 || offset + length > fBuffer.length() ) {
            throw new StringIndexOutOfBoundsException (
                "Invalid offset (" + offset + ") or length (" + length + ")" );
        }
        if ( length == 0 ) {
            return;
        }
        final StyleRun lastRun = getLastRun();
        if ( lastRun == null || lastRun.offset <= offset ) {
            Styler lastStyler = lastRun == null ? null : lastRun.style;
            appendStyleRun ( styler, offset );
            if ( offset + length != fBuffer.length() ) {
                appendStyleRun ( lastStyler, offset + length );
            }
            return;
        }
        int endRun = findRun ( offset + length );
        if ( endRun >= 0 ) {
        } else {
            endRun = - ( endRun + 1 );
            if ( offset + length < fBuffer.length() ) {
                Styler prevStyle = endRun > 0 ? fStyleRuns.getRun ( endRun - 1 ).style
                                   : null;
                fStyleRuns
                .add ( endRun, new StyleRun ( offset + length, prevStyle ) );
            }
        }
        int startRun = findRun ( offset );
        if ( startRun >= 0 ) {
            StyleRun styleRun = fStyleRuns.getRun ( startRun );
            styleRun.style = styler;
        } else {
            startRun = - ( startRun + 1 );
            Styler prevStyle = startRun > 0 ? fStyleRuns.getRun ( startRun - 1 ).style
                               : null;
            if ( isDifferentStyle ( prevStyle, styler )
                    || ( startRun == 0 && styler != null ) ) {
                fStyleRuns.add ( startRun, new StyleRun ( offset, styler ) );
                endRun++;
            } else {
                startRun--;
            }
        }
        if ( startRun + 1 < endRun ) {
            fStyleRuns.removeRange ( startRun + 1, endRun );
        }
    }
    public StyleRange[] getStyleRanges() {
        if ( hasRuns() ) {
            ArrayList res = new ArrayList();
            List styleRuns = getStyleRuns();
            int offset = 0;
            Styler style = null;
            for ( int i = 0; i < styleRuns.size(); i++ ) {
                StyleRun curr = ( StyleRun ) styleRuns.get ( i );
                if ( isDifferentStyle ( curr.style, style ) ) {
                    if ( curr.offset > offset && style != null ) {
                        res.add ( createStyleRange ( offset, curr.offset, style ) );
                    }
                    offset = curr.offset;
                    style = curr.style;
                }
            }
            if ( fBuffer.length() > offset && style != null ) {
                res.add ( createStyleRange ( offset, fBuffer.length(), style ) );
            }
            return ( StyleRange[] ) res.toArray ( new StyleRange[res.size()] );
        }
        return EMPTY;
    }
    private int findRun ( int offset ) {
        int low = 0;
        int high = fStyleRuns.size() - 1;
        while ( low <= high ) {
            int mid = ( low + high ) / 2;
            StyleRun styleRun = fStyleRuns.getRun ( mid );
            if ( styleRun.offset < offset ) {
                low = mid + 1;
            } else if ( styleRun.offset > offset ) {
                high = mid - 1;
            } else {
                return mid;
            }
        }
        return - ( low + 1 );
    }
    private StyleRange createStyleRange ( int start, int end, Styler style ) {
        StyleRange styleRange = new StyleRange();
        styleRange.start = start;
        styleRange.length = end - start;
        style.applyStyles ( styleRange );
        return styleRange;
    }
    private boolean hasRuns() {
        return fStyleRuns != null && !fStyleRuns.isEmpty();
    }
    private void appendStyleRun ( Styler style, int offset ) {
        StyleRun lastRun = getLastRun();
        if ( lastRun != null && lastRun.offset == offset ) {
            lastRun.style = style;
            return;
        }
        if ( lastRun == null && style != null || lastRun != null
                && isDifferentStyle ( style, lastRun.style ) ) {
            getStyleRuns().add ( new StyleRun ( offset, style ) );
        }
    }
    private boolean isDifferentStyle ( Styler style1, Styler style2 ) {
        if ( style1 == null ) {
            return style2 != null;
        }
        return !style1.equals ( style2 );
    }
    private StyleRun getLastRun() {
        if ( fStyleRuns == null || fStyleRuns.isEmpty() ) {
            return null;
        }
        return fStyleRuns.getRun ( fStyleRuns.size() - 1 );
    }
    private StyleRunList getStyleRuns() {
        if ( fStyleRuns == null ) {
            fStyleRuns = new StyleRunList();
        }
        return fStyleRuns;
    }
    private static class StyleRun {
        public int offset;
        public Styler style;
        public StyleRun ( int offset, Styler style ) {
            this.offset = offset;
            this.style = style;
        }
        public String toString() {
            return "Offset " + offset + ", style: " + style;
        }
    }
    private static class StyleRunList extends ArrayList {
        private static final long serialVersionUID = 123L;
        public StyleRunList() {
            super ( 3 );
        }
        public StyleRun getRun ( int index ) {
            return ( StyleRun ) get ( index );
        }
        public void removeRange ( int fromIndex, int toIndex ) {
            super.removeRange ( fromIndex, toIndex );
        }
    }
    private static class DefaultStyler extends Styler {
        private final String fForegroundColorName;
        private final String fBackgroundColorName;
        public DefaultStyler ( String foregroundColorName,
                               String backgroundColorName ) {
            fForegroundColorName = foregroundColorName;
            fBackgroundColorName = backgroundColorName;
        }
        public void applyStyles ( TextStyle textStyle ) {
            ColorRegistry colorRegistry = JFaceResources.getColorRegistry();
            if ( fForegroundColorName != null ) {
                textStyle.foreground = colorRegistry.get ( fForegroundColorName );
            }
            if ( fBackgroundColorName != null ) {
                textStyle.background = colorRegistry.get ( fBackgroundColorName );
            }
        }
    }
}
