public class MapFilter<T> implements Map<String, T> {
    final private Map<String, T> map;
    final private Map<String, Map.Entry<String, T>> entries = new TreeMap<String, Map.Entry<String, T>>();
    final private String prefix;
    public MapFilter ( Map<String, T> map, String prefix ) {
        this.map = map;
        this.prefix = prefix;
        rebuildEntries();
    }
    public MapFilter ( Map<String, T> map ) {
        this ( map, "" );
    }
    private synchronized void rebuildEntries() {
        entries.clear();
        for ( Map.Entry<String, T> e : map.entrySet() ) {
            String key = e.getKey();
            if ( key.startsWith ( prefix ) ) {
                String k = key.substring ( prefix.length() );
                entries.put ( k, e );
            }
        }
    }
    @Override
    public String toString() {
        StringBuilder s = new StringBuilder ( "MapFilter (" + prefix + ") of " + map + " containing " ).append ( entrySet() );
        return s.toString();
    }
    public MapFilter ( Properties p, String prefix ) {
        this ( new HashMap<String, T> ( ( Map ) p ), prefix );
    }
    public MapFilter<T> filter ( String prefix ) {
        return new MapFilter<T> ( this, prefix );
    }
    public int size() {
        return entries.size();
    }
    public boolean isEmpty() {
        return entries.isEmpty();
    }
    public boolean containsKey ( Object key ) {
        return entries.containsKey ( key );
    }
    public boolean containsValue ( Object value ) {
        for ( Map.Entry<String, T> e : entries.values() ) {
            if ( value.equals ( e.getValue() ) ) {
                return true;
            }
        }
        return false;
    }
    public T get ( Object key ) {
        return get ( key, null );
    }
    public T get ( Object key, T dflt ) {
        Map.Entry<String, T> e = entries.get ( ( String ) key );
        return e != null ? e.getValue() : dflt;
    }
    public T put ( String key, T value ) {
        T old = null;
        Map.Entry<String, T> entry = entries.get ( key );
        if ( entry != null ) {
            old = entry.setValue ( value );
        } else {
            map.put ( prefix + key, value );
            rebuildEntries();
        }
        return old;
    }
    public T remove ( Object key ) {
        Map.Entry<String, T> entry = entries.get ( ( String ) key );
        if ( entry != null ) {
            entries.remove ( key );
            return map.remove ( prefix + key );
        }
        return null;
    }
    public void putAll ( Map<? extends String, ? extends T> m ) {
        for ( Map.Entry<? extends String, ? extends T> e : m.entrySet() ) {
            put ( e.getKey(), e.getValue() );
        }
    }
    public void clear() {
        for ( String key : entries.keySet() ) {
            map.remove ( prefix + key );
        }
        entries.clear();
    }
    public Set<String> keySet() {
        return entries.keySet();
    }
    public Collection<T> values() {
        List<T> values = new ArrayList<T>();
        for ( Map.Entry<String, T> v : entries.values() ) {
            values.add ( v.getValue() );
        }
        return values;
    }
    public Set<Map.Entry<String, T>> entrySet() {
        Set<Map.Entry<String, T>> entrySet = new TreeSet<Map.Entry<String, T>>();
        for ( Map.Entry<String, Map.Entry<String, T>> v : entries.entrySet() ) {
            entrySet.add ( new Entry<T> ( v ) );
        }
        return entrySet;
    }
    private static class Entry<T> implements Map.Entry<String, T>, Comparable<Entry<T>> {
        private final Map.Entry<String, Map.Entry<String, T>> entry;
        Entry ( Map.Entry<String, Map.Entry<String, T>> entry ) {
            this.entry = entry;
        }
        public String getKey() {
            return entry.getKey();
        }
        public T getValue() {
            return entry.getValue().getValue();
        }
        public T setValue ( T newValue ) {
            return entry.getValue().setValue ( newValue );
        }
        @Override
        public boolean equals ( Object o ) {
            if ( ! ( o instanceof Entry ) ) {
                return false;
            }
            Entry e = ( Entry ) o;
            return getKey().equals ( e.getKey() ) && getValue().equals ( e.getValue() );
        }
        @Override
        public int hashCode() {
            return getKey().hashCode() ^ getValue().hashCode();
        }
        @Override
        public String toString() {
            return getKey() + "=" + getValue();
        }
        public int compareTo ( Entry<T> o ) {
            return getKey().compareTo ( o.getKey() );
        }
    }
    public static void main ( String[] args ) {
        String[] samples = {
            "Some.String.For.Me",
            "Some.String.For.You",
            "Some.More",
            "Yet.More"
        };
        Map map = new HashMap();
        for ( String s : samples ) {
            map.put ( s, s );
        }
        Map all = new MapFilter ( map );
        Map some = new MapFilter ( map, "Some." );
        Map someString = new MapFilter ( some, "String." );
        System.out.println ( "All: " + all );
        System.out.println ( "Some: " + some );
        System.out.println ( "Some.String: " + someString );
    }
}
