ConnectionPool original = new ConnectionPool ( String dbpoolName, ... );
TrackingConnectionPool trackingCP = new TrackingConnectionPool ( original );
TrackingConnectionPool trackingCP = new TrackingConnectionPool ( dbpoolName, ... );
public class TrackingConnectionPool extends ConnectionPool {
    private ConnectionPool originalPool;
    public TrackingConnectionPool ( String dbpoolName, ... ) {
        originalPool = new ConnectionPool ( dbpoolName, ... );
    }
    public TrackingConnectionPool ( ConnectionPool pool ) {
        this.originalPool = pool;
    }
    @Override
    public Connection getConnection() throws SQLException {
        Connection con = originalPool.getConnection();
        return new TrackableConnection ( con );
    }
    @Override
    public Connection getConnection ( long timeout ) throws SQLException {
        Connection con = originalPool.getConnection ( timeout );
        return new TrackableConnection ( con );
    }
    @Override
    public void setCaching ( boolean b ) {
        originalPool.setCaching ( b );
    }
    ...
}
public class TrackableConnection implements Connection, Tracker.Trackable {
    private Connection originalConnection;
    private boolean released = false;
    public TrackableConnection ( Connection con ) {
        this.originalConnection = con;
        Tracker.resourceAquired ( this );
    }
    @Override
    public boolean isReleased() {
        return this.released;
    }
    @Override
    public void release() {
        if ( !released ) {
            try {
                originalConnection.close();
                this.released = true;
            } catch ( SQLException e ) {
                throw new RuntimeException ( e );
            }
        }
    }
    @Override
    public void close() throws SQLException {
        originalConnection.close();
        this.released = true;
        Tracker.resourceReleased ( this );
    }
    @Override
    public Statement createStatement() throws SQLException {
        return originalConnection.createStatement();
    }
    ....
}
public class Tracker {
    private static final ThreadLocal<Tracker> _tracker = new ThreadLocal<Tracker>() {
        @Override
        protected Tracker initialValue() {
            return new Tracker();
        };
    };
    public interface Trackable {
        boolean isReleased();
        void release();
    }
    private Map<Trackable, Trackable> monitoredResources = new HashMap<Trackable, Trackable>();
    public static void start() {
        Tracker monitor = _tracker.get();
        monitor.monitoredResources.clear();
    }
    public static void stop() {
        Tracker monitor = _tracker.get();
        if ( !monitor.monitoredResources.isEmpty() ) {
            for ( Iterator<Trackable> it = monitor.monitoredResources.keySet().iterator(); it.hasNext(); ) {
                Trackable resource = it.next();
                if ( !resource.isReleased() ) {
                    System.out.println ( "WARNING: resource " + resource + " has not been released. Releasing it now." );
                    resource.release();
                } else {
                    System.out.println ( "Trackable " + resource
                                         + " is released but is still under monitoring. Perhaps you forgot to call resourceReleased()?" );
                }
            }
            monitor.monitoredResources.clear();
        }
    }
    public static void resourceAquired ( Trackable resource ) {
        Tracker monitor = _tracker.get();
        monitor.monitoredResources.put ( resource, resource );
    }
    public static void resourceReleased ( Trackable resource ) {
        Tracker monitor = _tracker.get();
        monitor.monitoredResources.remove ( resource );
    }
}
