package org.apache.accumulo.core.iterators;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import org.apache.accumulo.core.client.IteratorSetting;
import org.apache.accumulo.core.data.ByteSequence;
import org.apache.accumulo.core.data.Key;
import org.apache.accumulo.core.data.PartialKey;
import org.apache.accumulo.core.data.Range;
import org.apache.accumulo.core.data.Value;
import org.apache.hadoop.io.Text;
public class FirstEntryInRangeIterator extends SkippingIterator implements OptionDescriber {
    static final String NUM_SCANS_STRING_NAME = "scansBeforeSeek";
    private Range latestRange;
    private Collection<ByteSequence> latestColumnFamilies;
    private boolean latestInclusive;
    private Text lastRowFound;
    private int numscans;
    public static void setNumScansBeforeSeek ( IteratorSetting cfg, int num ) {
        cfg.addOption ( NUM_SCANS_STRING_NAME, Integer.toString ( num ) );
    }
    public FirstEntryInRangeIterator() {
        super();
    }
    public FirstEntryInRangeIterator ( FirstEntryInRangeIterator other, IteratorEnvironment env ) {
        super();
        setSource ( other.getSource().deepCopy ( env ) );
    }
    @Override
    public SortedKeyValueIterator<Key, Value> deepCopy ( IteratorEnvironment env ) {
        return new FirstEntryInRangeIterator ( this, env );
    }
    @Override
    public void init ( SortedKeyValueIterator<Key, Value> source, Map<String, String> options, IteratorEnvironment env ) throws IOException {
        super.init ( source, options, env );
        String o = options.get ( NUM_SCANS_STRING_NAME );
        numscans = o == null ? 10 : Integer.parseInt ( o );
    }
    @Override
    protected void consume() throws IOException {
        if ( finished == true || lastRowFound == null ) {
            return;
        }
        int count = 0;
        while ( getSource().hasTop() && lastRowFound.equals ( getSource().getTopKey().getRow() ) ) {
            if ( count < numscans ) {
                ++count;
                getSource().next();
            } else {
                count = 0;
                Key nextKey = getSource().getTopKey().followingKey ( PartialKey.ROW );
                if ( !latestRange.afterEndKey ( nextKey ) ) {
                    getSource().seek ( new Range ( nextKey, true, latestRange.getEndKey(), latestRange.isEndKeyInclusive() ), latestColumnFamilies, latestInclusive );
                } else {
                    finished = true;
                    break;
                }
            }
        }
        lastRowFound = getSource().hasTop() ? getSource().getTopKey().getRow ( lastRowFound ) : null;
    }
    private boolean finished = true;
    @Override
    public boolean hasTop() {
        return !finished && getSource().hasTop();
    }
    @Override
    public void seek ( Range range, Collection<ByteSequence> columnFamilies, boolean inclusive ) throws IOException {
        latestRange = range;
        latestColumnFamilies = columnFamilies;
        latestInclusive = inclusive;
        lastRowFound = null;
        super.seek ( range, columnFamilies, inclusive );
        finished = false;
        if ( getSource().hasTop() ) {
            lastRowFound = getSource().getTopKey().getRow();
            if ( range.beforeStartKey ( getSource().getTopKey() ) ) {
                consume();
            }
        }
    }
    @Override
    public IteratorOptions describeOptions() {
        String name = "firstEntry";
        String desc = "Only allows iteration over the first entry per range";
        HashMap<String, String> namedOptions = new HashMap<String, String>();
        namedOptions.put ( NUM_SCANS_STRING_NAME, "Number of scans to try before seeking [10]" );
        return new IteratorOptions ( name, desc, namedOptions, null );
    }
    @Override
    public boolean validateOptions ( Map<String, String> options ) {
        try {
            String o = options.get ( NUM_SCANS_STRING_NAME );
            if ( o != null ) {
                Integer.parseInt ( o );
            }
        } catch ( Exception e ) {
            throw new IllegalArgumentException ( "bad integer " + NUM_SCANS_STRING_NAME + ":" + options.get ( NUM_SCANS_STRING_NAME ), e );
        }
        return true;
    }
}
