import com.google.common.collect.Lists;
import com.google.common.util.concurrent.MoreExecutors;
import java.util.Iterator;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
public class ThreadedIteratorProcessor<T> {
    public static interface ElementProcessor<T> {
        public void processElement ( T element );
    }
    private final int numThreads;
    public ThreadedIteratorProcessor ( int numThreads ) {
        this.numThreads = numThreads;
    }
    public ThreadedIteratorProcessor() {
        this ( Runtime.getRuntime().availableProcessors() );
    }
    public void processIterator ( Iterator<T> iterator, ElementProcessor<T> elementProcessor ) {
        ExecutorService e = Executors.newFixedThreadPool ( numThreads, MoreExecutors.daemonThreadFactory() );
        List<Future<?>> futures = Lists.newLinkedList();
        final SafeIterator<T> safeIterator = new SafeIterator<T> ( iterator );
        for ( int i = 0; i < numThreads; i++ ) {
            futures.add ( e.submit ( new Consumer<T> ( safeIterator, elementProcessor ) ) );
        }
        e.shutdown();
        for ( Future<?> future : futures ) {
            try {
                future.get();
            } catch ( InterruptedException ex ) {
            } catch ( ExecutionException ex ) {
                throw Throwables.propagate ( ex.getCause() );
            }
        }
    }
    private static class Consumer<T> implements Runnable {
        private final SafeIterator<T> it;
        private final ElementProcessor<T> elementProcessor;
        public Consumer ( SafeIterator<T> it, ElementProcessor<T> elementProcessor ) {
            this.it = it;
            this.elementProcessor = elementProcessor;
        }
        @Override
        public void run() {
            while ( true ) {
                T o = it.nextOrNull();
                if ( o == null ) {
                    return;
                }
                elementProcessor.processElement ( o );
            }
        }
    }
    private static class SafeIterator<T> {
        private final Iterator<T> in;
        SafeIterator ( Iterator<T> in ) {
            this.in = in;
        }
        synchronized T nextOrNull() {
            if ( in.hasNext() ) {
                return in.next();
            }
            return null;
        }
    }
}
