private static Map<String, ColumnMappingDTO> initColumnMappings ( String pInputClassName ) throws Exception {
    Map<String, ColumnMappingDTO> outMappingDetails = new HashMap<String, ColumnMappingDTO>();
    try {
        for ( Field vField : getFields ( pInputClassName ) ) {
            for ( Annotation vAnnotation : vField.getDeclaredAnnotations() ) {
                if ( vAnnotation instanceof Column ) {
                    ColumnMappingDTO vColumnMappingDTO = createColumnMapping ( vField, ( Column ) vAnnotation );
                    outMappingDetails.put ( vColumnMappingDTO.getColumnName(), vColumnMappingDTO );
                }
            }
        }
    } catch ( Exception ex ) {
        System.out.println ( "caught Exception in initColumnMappings() as  " + ex );
        throw ex;
    }
    return outMappingDetails;
}
private static ColumnMappingDTO createColumnMapping ( Field vField, Column vAnnotation ) {
    ColumnMappingDTO vColumnMappingDTO = new ColumnMappingDTO();
    vColumnMappingDTO.setPropertyName ( vField.getName() );
    vColumnMappingDTO.setColumnName ( vAnnotation.name() );
    return vColumnMappingDTO;
}
private static Field[] getFields ( String pInputClassName ) throws ClassNotFoundException {
    return Class.forName ( pInputClassName ).getDeclaredFields();
}
public static Object executeQuery ( PreparedStatement inPstmt, String pDomainClassName ) throws SQLException, Exception {
    return executeQuery ( inPstmt, Class.forName ( pDomainClassName ) );
}
public static <T> List<T>  executeQuery ( PreparedStatement inPstmt, Class<T> domainClass ) throws SQLException, Exception {
    List<T> outResultList = new ArrayList<T>();
    ResultSet mRSet = null;
    try {
        Map<String, ColumnMappingDTO> mMappingDetailsMap = initColumnMappings ( domainClass.getName() );
        mRSet = inPstmt.executeQuery();
        if ( mRSet == null ) {
            return Collections.EMPTY_LIST;
        }
        List<String> mColumnNamesList = getColumnNameList ( mRSet );
        while ( mRSet.next() ) {
            List<ColumnValueDTO> mColumnValuesList = new ArrayList<ColumnValueDTO>();
            for ( String columnHeader : mColumnNamesList ) {
                if ( mMappingDetailsMap.containsKey ( columnHeader ) ) {
                    ColumnValueDTO mColumnValueDTO = new ColumnValueDTO();
                    mColumnValueDTO.setPropertyName ( mMappingDetailsMap.get ( columnHeader ).getPropertyName() );
                    mColumnValueDTO.setPropertyValue ( mRSet.getString ( mMappingDetailsMap.get ( columnHeader ).getColumnName() ) );
                    mColumnValuesList.add ( mColumnValueDTO );
                }
            }
            T domainObj = createDomainObject ( mColumnValuesList, domainClass );
            outResultList.add ( domainObj );
        }
    } catch ( Exception ex ) {
        System.out.println ( " caught in executeQuery() " + ex );
        throw ex;
    } finally {
        try {
            mRSet.close();
        } catch ( SQLException e ) {
            System.out.println ( " caught in Exception while closing ResultSet " + e );
            throw e;
        }
    }
    return outResultList;
}
private static List<String> getColumnNameList ( ResultSet mRSet ) throws SQLException {
    ResultSetMetaData mRsetMt = mRSet.getMetaData();
    if ( mRsetMt == null ) {
        return Collections.EMPTY_LIST;
    }
    List<String> mColumnNamesList = new ArrayList<String>();
    for ( int i = 0; i < mRsetMt.getColumnCount(); i++ ) {
        mColumnNamesList.add ( mRsetMt.getColumnName ( i + 1 ) );
    }
    return mColumnNamesList;
}
private static Object createDomainObject ( List<ColumnValueDTO> columnValuesList, String vDoaminClass ) throws Exception {
    Class<?> domainClassObj = Class.forName ( vDoaminClass );
    return createDomainObject ( columnValuesList, domainClassObj );
}
private static <T> T createDomainObject ( List<ColumnValueDTO> columnValuesList, Class<T> domainClassObj ) throws Exception {
    T domainObj = null;
    try {
        domainObj = domainClassObj.newInstance();
        for ( ColumnValueDTO columnDTO : columnValuesList ) {
            if ( columnDTO == null ) {
                continue;
            }
            Field domainDataField = domainClassObj.getDeclaredField ( columnDTO.getPropertyName() );
            domainDataField.setAccessible ( true );
            Object valueByType = parseValueByType ( columnDTO.getPropertyValue(), domainDataField.getType() );
            domainDataField.set ( domainObj, valueByType );
        }
    } catch ( Exception e ) {
        System.out.println ( " Caught " + e.getClass().getSimpleName() + " in createDomainObject() " + e );
        throw e;
    }
    return domainObj;
}
private static Object parseValueByType ( String value, Type type ) throws Exception {
    if ( value == null ) {
        return null;
    }
    try {
        if ( Integer.TYPE.equals ( type ) ) {
            return Integer.parseInt ( value );
        }
        if ( Double.TYPE.equals ( type ) ) {
            return Double.parseDouble ( value );
        }
        if ( Float.TYPE.equals ( type ) ) {
            return Float.parseFloat ( value );
        }
        if ( Short.TYPE.equals ( type ) ) {
            return Short.parseShort ( value );
        }
        if ( Long.TYPE.equals ( type ) ) {
            return Long.parseLong ( value );
        }
        if ( java.sql.Timestamp.class.equals ( type ) ) {
            return java.sql.Timestamp.valueOf ( value );
        }
        if ( java.sql.Date.class.equals ( type ) ) {
            return java.sql.Date.valueOf ( value );
        }
        if ( String.class.equals ( type ) ) {
            return value;
        }
        if ( Character.TYPE.equals ( type ) ) {
            if ( value.length() == 1 ) {
                return value.charAt ( 0 );
            }
            if ( value.length() == 0 ) {
                return '\0';
            }
            throw new IllegalStateException ( "" );
        }
    } catch ( Exception ex ) {
        System.out.println ( " Caught Exception in getValueByType() " + ex );
        throw ex;
    }
    throw new IllegalArgumentException ( "Could not find the resolver for type " + type );
}
public static class ColumnValueDTO implements Serializable {
    private String propertyName;
    private String propertyValue;
    private static final long serialVersionUID = -4915109169715618102L;
    public String getPropertyName() {
        return propertyName;
    }
    public void setPropertyName ( String propertyName ) {
        this.propertyName = propertyName;
    }
    public String getPropertyValue() {
        return propertyValue;
    }
    public void setPropertyValue ( String propertyValue ) {
        this.propertyValue = propertyValue;
    }
    public String toString() {
        return "Property ::  " + this.propertyName + "   Value :: " + this.propertyValue;
    }
}
public static class ColumnMappingDTO {
    private String columnName;
    private String propertyName;
    private String dataType;
    public String getColumnName() {
        return columnName;
    }
    public void setColumnName ( String columnName ) {
        this.columnName = columnName;
    }
    public String getDataType() {
        return dataType;
    }
    public void setDataType ( String dataType ) {
        this.dataType = dataType;
    }
    public String getPropertyName() {
        return propertyName;
    }
    public void setPropertyName ( String propertyName ) {
        this.propertyName = propertyName;
    }
    public String toString() {
        return "Database Column :: " + this.columnName + "  Java Property :: " + this.propertyName + "   Java Datatype :: " + this.dataType;
    }
}
private static class Column {
    public String name() {
        return null;
    }
}
