public class Base32 {
    private static final String base32HexCharacterSet = "0123456789ABCDEFGHIJKLMNOPQRSTUV";
    private static final String base32CharacterSet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ234567";
    private static final String zBase32CharacterSet = "YBNDRFG8EJKMCPQXOT1UWISZA345H769";
    private static final String crockfordCharacterSet = "0123456789ABCDEFGHJKMNPQRSTVWXYZ";
    public static final Base32 ordinary = new Base32();
    public static final Base32 base32Hex = new Base32 ( base32HexCharacterSet );
    public static final Base32 base32 = new Base32 ( base32CharacterSet );
    public static final Base32 zBase32 = new Base32 ( zBase32CharacterSet );
    public static final Base32 crockfords = new Base32 ( crockfordCharacterSet, "O0", "o0", "L1", "l1", "I1", "i1" );
    private static final int Invalid = -1;
    private static final int radix = 32;
    private static final int bitsPerDigit = 5;
    private static final int bitsPerByte = 8;
    private final char[] formatTable;
    private final int[] parseTable;
    public Base32() {
        formatTable = null;
        parseTable = null;
    }
    protected Base32 ( String characterSet, String... extras ) {
        if ( characterSet.length() != radix ) {
            throw new NumberFormatException ( "Invalid character set. Must be " + radix + " long" );
        }
        formatTable = buildFormatTable ( characterSet );
        parseTable = buildParseTable ( characterSet, extras );
    }
    private char[] buildFormatTable ( String characterSet ) {
        char[] table = new char[radix];
        for ( int i = 0; i < radix; i++ ) {
            table[i] = characterSet.charAt ( i );
        }
        return table;
    }
    private int[] buildParseTable ( String characterSet, String... extras ) {
        int[] table = new int['z' + 1];
        Arrays.fill ( table, Invalid );
        String lc = characterSet.toLowerCase();
        String uc = characterSet.toUpperCase();
        for ( int i = 0; i < radix; i++ ) {
            char l = lc.charAt ( i );
            char u = uc.charAt ( i );
            if ( table[l] == Invalid && table[u] == Invalid ) {
                table[l] = i;
                table[u] = i;
            } else {
                throw new NumberFormatException ( "Invalid character set - duplicate found at position " + i );
            }
        }
        for ( String pair : extras ) {
            if ( pair.length() == 2 ) {
                int f = pair.charAt ( 1 );
                int t = pair.charAt ( 0 );
                if ( table[f] != Invalid && table[t] == Invalid ) {
                    table[t] = table[f];
                } else {
                    throw new NumberFormatException ( "Invalid character set extra - copying from " + f + " to " + t );
                }
            } else {
                throw new NumberFormatException ( "Invalid extra \"" + pair + "\" - should be 2 characters wide." );
            }
        }
        return table;
    }
    public String format ( BigInteger n ) {
        String formatted = n.toString ( radix ).toUpperCase();
        if ( formatTable != null ) {
            char[] translated = new char[formatted.length()];
            for ( int i = 0; i < formatted.length(); i++ ) {
                int d = Character.digit ( formatted.charAt ( i ), radix );
                translated[i] = formatTable[d];
            }
            formatted = new String ( translated );
        }
        return formatted;
    }
    public BigInteger parse ( String s ) {
        BigInteger big;
        if ( parseTable != null ) {
            int digits = s.length();
            int bits = digits * bitsPerDigit + 1;
            int bytes = ( bits + bitsPerByte - 1 ) / bitsPerByte;
            int bias = ( bytes * bitsPerByte ) - bits + 1;
            byte[] parsed = new byte[bytes];
            for ( int i = 0, bit = bias; i < digits; i++, bit += bitsPerDigit ) {
                char c = s.charAt ( i );
                if ( c < parseTable.length ) {
                    int n = parseTable[c];
                    switch ( n ) {
                    case 0:
                        break;
                    default:
                        int shift = ( bitsPerByte - bitsPerDigit - ( bit % bitsPerByte ) );
                        int bite = bit / bitsPerByte;
                        if ( shift >= 0 ) {
                            parsed[bite] |= n << shift;
                        } else {
                            parsed[bite] |= n >>> -shift;
                            parsed[bite + 1] |= n << ( bitsPerByte + shift );
                        }
                        break;
                    case Invalid:
                        throw new NumberFormatException ( "Invalid character '" + c + "' at position " + i );
                    }
                } else {
                    throw new NumberFormatException ( "Invalid character '" + c + "' at position " + i );
                }
            }
            big = new BigInteger ( parsed );
        } else {
            big = new BigInteger ( s, radix );
        }
        return big;
    }
    public boolean good ( String s ) {
        boolean good = true;
        for ( int i = 0; i < s.length() && good; i++ ) {
            char c = s.charAt ( i );
            if ( parseTable != null ) {
                if ( c < parseTable.length ) {
                    good = parseTable[c] != Invalid;
                } else {
                    good = false;
                }
            } else {
                good = Character.digit ( c, radix ) != -1;
            }
        }
        return good;
    }
    public static void main ( String args[] ) {
        Test.main ( args );
    }
}
class Test {
    private static Random r = new Random();
    private static final int testBits = 95;
    public static void main ( String args[] ) {
        test ( new BigInteger ( "10" ) );
        test ( new BigInteger ( "32" ) );
        test ( new BigInteger ( "100" ) );
        BigInteger big = BigInteger.valueOf ( 0 );
        for ( int i = 0; i < 1000; i++, big = big.add ( BigInteger.ONE ) ) {
            test ( big );
        }
        for ( int i = 0; i < 1000; i++ ) {
            test ( new BigInteger ( testBits, r ) );
        }
        testCrockfords();
    }
    private static void test ( BigInteger i ) {
        test ( i, Base32.ordinary, "Ordinary" );
        test ( i, Base32.base32Hex, "Base32Hex" );
        test ( i, Base32.base32, "Base32" );
        test ( i, Base32.crockfords, "Crockfords" );
        test ( i, Base32.zBase32, "ZBase32" );
    }
    private static void test ( BigInteger i, Base32 f, String name ) {
        test ( i, f, f.format ( i ), name );
    }
    private static void test ( BigInteger i, Base32 f, String formatted, String name ) {
        BigInteger parsed = f.parse ( formatted );
        boolean ok = parsed.equals ( i ) && f.good ( formatted );
        BigInteger reParsed = f.parse ( formatted );
        boolean good = f.good ( formatted );
        System.out.println ( i + " = " + f.format ( i ) + " in " + name + ( ok ? " Ok" : " BAD!" ) );
        if ( !ok ) {
            System.out.println ( reParsed + " != " + i );
        }
    }
    private static void testCrockfords() {
        BigInteger anztenney = new BigInteger ( "0003019140802085400304608040952" );
        String formatted = Base32.crockfords.format ( anztenney );
        test ( anztenney, Base32.crockfords, formatted, "Crockfords Test" );
        for ( int i = 0; i < 100; i++ ) {
            BigInteger b = BigInteger.valueOf ( i );
            formatted = Base32.crockfords.format ( b );
            test ( b, Base32.crockfords, formatted, "Crockfords Test" );
        }
        for ( int i = 0; i < 1000; i++ ) {
            BigInteger b = new BigInteger ( testBits, r );
            formatted = Base32.crockfords.format ( b )
                        .replace ( '0', 'O' )
                        .replace ( '1', 'l' );
            test ( b, Base32.crockfords, formatted, "Crockfords Test" );
        }
    }
}
