import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Arrays;
public class BitmapEncoder {
    private static final int BMP_SIZE_HEADER = 54;
    private static final int BMP_SIZE_IMAGE_WIDTH          = 4;
    private static final int BMP_SIZE_PAYLOAD_LENGTH       = 4;
    private static final int BMP_SIZE_BMPUTIL_MAGIC        = 4;
    private static final int BMP_OFFSET_FILESIZE_BYTES     = 2;
    private static final int BMP_OFFSET_IMAGE_WIDTH        = 18;
    private static final int BMP_OFFSET_IMAGE_HEIGHT       = 22;
    private static final int BMP_OFFSET_IMAGE_DATA_BYTES   = 34;
    private static final int BMP_OFFSET_PAYLOAD_LENGTH     = 38;
    private static final int BMP_OFFSET_BMPUTIL_MAGIC      = 42;
    private static final byte UDEF = 0;
    private static final byte[] BMP_HEADER = new byte[] {
        0x42, 0x4d,
        UDEF, UDEF, UDEF, UDEF,
        0x00, 0x00,
        0x00, 0x00,
        0x36, 0x00, 0x00, 0x00,
        0x28, 0x00, 0x00, 0x00,
        UDEF, UDEF, UDEF, UDEF,
        UDEF, UDEF, UDEF, UDEF,
        0x01, 0x00,
        0x18, 0x00,
        0x00, 0x00, 0x00, 0x00,
        UDEF, UDEF, UDEF, UDEF,
        UDEF, UDEF, UDEF, UDEF,
        UDEF, UDEF, UDEF, UDEF,
        0x00, 0x00, 0x00, 0x00,
        0x00, 0x00, 0x00, 0x00,
    };
    private static final byte[] BMPUTIL_MAGIC = new byte[] {
        0x30, 0x32, 0x30, 0x35
    };
    public static void encodeToBitmap ( File srcFile, File destFile ) throws IOException {
        encodeToBitmap ( new FileInputStream ( srcFile ), srcFile.length(), new   FileOutputStream ( destFile ) );
    }
    public static void encodeToBitmap ( byte[] srcBytes, File destFile ) throws IOException       {
        encodeToBitmap ( new ByteArrayInputStream ( srcBytes ), srcBytes.length, new  FileOutputStream ( destFile ) );
    }
    public static void encodeToBitmap ( byte[] srcBytes, OutputStream destStream ) throws  IOException {
        encodeToBitmap ( new ByteArrayInputStream ( srcBytes ), srcBytes.length, destStream );
    }
    public static void encodeToBitmap ( InputStream srcStream, long srcStreamLength,  OutputStream destStream ) throws IOException {
        if ( srcStreamLength > Integer.MAX_VALUE ) {
            throw new IOException ( "File too big; max. " + Integer.MAX_VALUE + " bytes supported." );
        }
        int imageWidth = ( int ) Math.ceil ( Math.sqrt ( ( double ) srcStreamLength / 3 ) );
        / / Image width, sqrt ( payload / 3 ), divided by 3 because of RGB
        int imageHeight = ( int ) Math.ceil ( ( double ) srcStreamLength
                                              / ( double ) imageWidth / 3 );
        int rowPadding = 4 - ( imageWidth * 3 % 4 );
        int filesizeBytes = imageWidth * imageHeight * 3 + imageHeight * rowPadding
                            + BMP_SIZE_HEADER;
        int imageBytesWithPadding = filesizeBytes - BMP_SIZE_HEADER;
        int payloadPadding = ( int ) ( imageWidth * imageHeight * 3 - srcStreamLength );
        byte[] header = BMP_HEADER.clone();
        writeIntLE ( header, BMP_OFFSET_FILESIZE_BYTES, filesizeBytes );
        writeIntLE ( header, BMP_OFFSET_IMAGE_WIDTH, imageWidth );
        writeIntLE ( header, BMP_OFFSET_IMAGE_HEIGHT, imageHeight );
        writeIntLE ( header, BMP_OFFSET_IMAGE_DATA_BYTES, imageBytesWithPadding );
        writeIntLE ( header, BMP_OFFSET_PAYLOAD_LENGTH, ( int ) srcStreamLength );
        System.arraycopy ( BMPUTIL_MAGIC, 0, header,
                           BMP_OFFSET_BMPUTIL_MAGIC, BMPUTIL_MAGIC.length );
        destStream.write ( header, 0, header.length );
        byte[] row = new byte[imageWidth * 3];
        int read;
        while ( ( read = srcStream.read ( row ) ) != -1 ) {
            destStream.write ( row, 0, read );
            destStream.write ( new byte[rowPadding] );
        }
        destStream.write ( new byte[payloadPadding] );
        srcStream.close();
        destStream.close();
    }
    public static byte[] decodeFromBitmap ( InputStream srcStream ) throws    IOException {
        ByteArrayOutputStream destStream = new ByteArrayOutputStream();
        decodeFromBitmap ( srcStream, destStream );
        return destStream.toByteArray();
    }
    public static byte[] decodeFromBitmap ( File srcFile ) throws IOException {
        ByteArrayOutputStream destStream = new ByteArrayOutputStream();
        decodeFromBitmap ( new FileInputStream ( srcFile ), destStream );
        return destStream.toByteArray();
    }
    public static void decodeFromBitmap ( File srcFile, File destFile ) throws  IOException {
        decodeFromBitmap ( new FileInputStream ( srcFile ), new  FileOutputStream ( destFile ) );
    }
    public static void decodeFromBitmap ( InputStream srcStream, OutputStream  destStream ) throws IOException {
        long bytesRead = 0;
        bytesRead += srcStream.skip ( BMP_OFFSET_IMAGE_WIDTH - bytesRead );
        byte[] imageWidthBytes = new byte[BMP_SIZE_IMAGE_WIDTH];
        bytesRead += srcStream.read ( imageWidthBytes );
        int imageWidth = toIntLE ( imageWidthBytes );
        bytesRead += srcStream.skip ( BMP_OFFSET_PAYLOAD_LENGTH - bytesRead );
        byte[] payloadLengthBytes = new byte[BMP_SIZE_PAYLOAD_LENGTH];
        bytesRead += srcStream.read ( payloadLengthBytes );
        int payloadLength = toIntLE ( payloadLengthBytes );
        bytesRead += srcStream.skip ( BMP_OFFSET_BMPUTIL_MAGIC - bytesRead );
        byte[] magicFieldBytes = new byte[BMP_SIZE_BMPUTIL_MAGIC];
        bytesRead += srcStream.read ( magicFieldBytes );
        if ( !Arrays.equals ( BMPUTIL_MAGIC, magicFieldBytes ) ) {
            throw new IOException ( "Given bitmap does not contain     encoded binary data." );
        }
        bytesRead += srcStream.skip ( BMP_SIZE_HEADER - bytesRead );
        if ( payloadLength > 0 ) {
            int rowPaddingLength = 4 - ( imageWidth * 3 % 4 );
            int rowLength = imageWidth * 3;
            byte[] row = new byte[rowLength];
            int read;
            int restOfPayload = payloadLength;
            while ( ( read = srcStream.read ( row ) ) != -1 ) {
                if ( restOfPayload >= read ) {
                    destStream.write ( row, 0, read );
                    srcStream.skip ( rowPaddingLength );
                    restOfPayload -= read;
                } else {
                    destStream.write ( row, 0, restOfPayload );
                    break;
                }
            }
        }
        srcStream.close();
        destStream.close();
    }
    private static void writeIntLE ( byte[] bytes, int startoffset, int            value ) {
        bytes[startoffset] = ( byte ) ( value );
        bytes[startoffset + 1] = ( byte ) ( value >>> 8 );
        bytes[startoffset + 2] = ( byte ) ( value >>> 16 );
        bytes[startoffset + 3] = ( byte ) ( value >>> 24 );
    }
    private static int toIntLE ( byte[] value ) {
        return ( ( value[3] & 0xff ) << 24 ) |
               ( ( value[2] & 0xff ) << 16 ) |
               ( ( value[1] & 0xff ) << 8 ) |
               ( value[0] & 0xff );
    }
    public static void main ( String[] args ) throws IOException  {
        if ( args.length >= 2 && "encode".equals ( args[0] ) ) {
            BitmapEncoder.encodeToBitmap ( new File ( args[1] ), new File ( args[2] ) );
        } else if ( args.length >= 2 && "decode".equals ( args[0] ) ) {
            BitmapEncoder.decodeFromBitmap ( new File ( args[1] ), new                           File ( args[2] ) );
        } else {
            System.out.println ( "Usage: BitmapEncoder encode SRCFILE   DESTFILE.bmp" );
            System.out.println ( "       BitmapEncoder decode SRCFILE.bmp DESTFILE" );
        }
    }
}
