import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.DataBuffer;
import java.awt.image.DirectColorModel;
import java.awt.image.SampleModel;
import java.awt.image.SinglePixelPackedSampleModel;
import java.awt.image.WritableRaster;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.SwingUtilities;
public class SparseBufferedImageTest {
    public static void main ( String[] args ) {
        SparseBufferedImage.PRINT_DATA_SIZE = true;
        SparseBufferedImage sbi = new SparseBufferedImage ( 1000, 1000 );
        Graphics2D g = sbi.createGraphics();
        g.setColor ( Color.BLACK );
        g.drawString ( "Test", 20, 20 );
        g.setColor ( Color.RED );
        g.fillOval ( 300, 300, 40, 40 );
        g.setColor ( Color.GREEN );
        g.fillRect ( 600, 700, 20, 20 );
        g.setColor ( Color.BLUE );
        g.drawLine ( 200, 800, 800, 200 );
        g.dispose();
        show ( sbi );
    }
    private static void show ( final BufferedImage b ) {
        SwingUtilities.invokeLater ( new Runnable() {
            @Override
            public void run() {
                JFrame f = new JFrame();
                f.setDefaultCloseOperation ( JFrame.EXIT_ON_CLOSE );
                f.getContentPane().add ( new JLabel ( new ImageIcon ( b ) ) );
                f.pack();
                f.setLocationRelativeTo ( null );
                f.setVisible ( true );
            }
        } );
    }
}
class SparseBufferedImage extends BufferedImage {
    public static boolean PRINT_DATA_SIZE = false;
    private static final int MASK_RGB_RED   = 0x00ff0000;
    private static final int MASK_RGB_GREEN = 0x0000ff00;
    private static final int MASK_RGB_BLUE  = 0x000000ff;
    private static final int MASK_BGR_RED   = 0x000000ff;
    private static final int MASK_BGR_GREEN = 0x0000ff00;
    private static final int MASK_BGR_BLUE  = 0x00ff0000;
    private static final int MASK_ARGB_ALPHA = 0xff000000;
    private static final int MASK_ARGB_RED   = 0x00ff0000;
    private static final int MASK_ARGB_GREEN = 0x0000ff00;
    private static final int MASK_ARGB_BLUE  = 0x000000ff;
    public SparseBufferedImage ( int width, int height ) {
        this ( width, height, BufferedImage.TYPE_INT_ARGB );
    }
    public SparseBufferedImage ( int width, int height, int type ) {
        super (
            createColorModel ( type ),
            createPackedRaster (
                new SparseDataBufferInt ( width * height ),
                width, height, createBandmasks ( type ) ),
            false, new Hashtable<Object, Object>() );
    }
    private static int[] createBandmasks ( int type ) {
        if ( type == BufferedImage.TYPE_INT_RGB ) {
            int bandmasks[] = new int[3];
            bandmasks[0] = MASK_RGB_RED;
            bandmasks[1] = MASK_RGB_GREEN;
            bandmasks[2] = MASK_RGB_BLUE;
            return bandmasks;
        } else if ( type == BufferedImage.TYPE_INT_BGR ) {
            int bandmasks[] = new int[3];
            bandmasks[0] = MASK_BGR_RED;
            bandmasks[1] = MASK_BGR_GREEN;
            bandmasks[2] = MASK_BGR_BLUE;
            return bandmasks;
        } else if ( type == BufferedImage.TYPE_INT_ARGB ) {
            int bandmasks[] = new int[4];
            bandmasks[0] = MASK_ARGB_RED;
            bandmasks[1] = MASK_ARGB_GREEN;
            bandmasks[2] = MASK_ARGB_BLUE;
            bandmasks[3] = MASK_ARGB_ALPHA;
            return bandmasks;
        }
        throw new IllegalArgumentException ( "Invalid image type: " + type );
    }
    private static ColorModel createColorModel ( int type ) {
        if ( type == BufferedImage.TYPE_INT_RGB ) {
            ColorModel colorModel = new DirectColorModel ( 24,
                    MASK_RGB_RED, MASK_RGB_GREEN, MASK_RGB_BLUE, 0x0 );
            return colorModel;
        } else if ( type == BufferedImage.TYPE_INT_BGR ) {
            ColorModel colorModel = new DirectColorModel ( 24,
                    MASK_BGR_RED, MASK_BGR_GREEN, MASK_BGR_BLUE, 0x0 );
            return colorModel;
        } else if ( type == BufferedImage.TYPE_INT_ARGB ) {
            ColorModel colorModel = new DirectColorModel ( 32,
                    MASK_ARGB_RED, MASK_ARGB_GREEN, MASK_ARGB_BLUE, MASK_ARGB_ALPHA );
            return colorModel;
        }
        throw new IllegalArgumentException ( "Invalid image type: " + type );
    }
    private static WritableRaster createPackedRaster (
        SparseDataBufferInt dataBuffer, int w, int h, int bandMasks[] ) {
        SinglePixelPackedSampleModel singlePixelPackedSampleModel =
            new SinglePixelPackedSampleModel (
            dataBuffer.getDataType(), w, h, w, bandMasks );
        return new SparseIntegerInterleavedRaster (
                   singlePixelPackedSampleModel, dataBuffer );
    }
    private static class SparseDataBufferInt extends DataBuffer {
        private SparseIntBuffer data;
        public SparseDataBufferInt ( int size ) {
            super ( TYPE_INT, size );
            data = new SparseIntBuffer();
        }
        SparseIntBuffer getData() {
            return data;
        }
        @Override
        public int getElem ( int i ) {
            return data.get ( i + offset );
        }
        @Override
        public int getElem ( int bank, int i ) {
            return data.get ( i + offsets[bank] );
        }
        @Override
        public void setElem ( int i, int val ) {
            data.put ( i + offset, val );
        }
        @Override
        public void setElem ( int bank, int i, int val ) {
            data.put ( i + offsets[bank], val );
        }
    }
    private static class SparseIntegerInterleavedRaster extends WritableRaster {
        private SparseDataBufferInt data;
        public SparseIntegerInterleavedRaster (
            SampleModel sampleModel, SparseDataBufferInt dataBuffer ) {
            super (
                sampleModel, dataBuffer,
                new Rectangle (
                    0, 0,
                    sampleModel.getWidth(),
                    sampleModel.getHeight() ),
                new Point ( 0, 0 ), null );
            this.data = dataBuffer;
        }
        @Override
        public Object getDataElements ( int x, int y, Object obj ) {
            int outData[];
            if ( obj == null ) {
                outData = new int[1];
            } else {
                outData = ( int[] ) obj;
            }
            int off = y * width + x;
            outData[0] = data.getData().get ( off );
            return outData;
        }
        @Override
        public void setDataElements ( int x, int y, Object obj ) {
            int inData[] = ( int[] ) obj;
            int off = y * width + x;
            data.getData().put ( off, inData[0] );
        }
    }
    private static class SparseIntBuffer {
        private final Map<Integer, Integer> map =
            new HashMap<Integer, Integer>();
        int get ( int index ) {
            Integer value = map.get ( index );
            if ( value == null ) {
                return 0;
            }
            return value;
        }
        void put ( int index, int value ) {
            map.put ( index, value );
            if ( PRINT_DATA_SIZE ) {
                System.out.println ( "Put " + value + " at " + index + ", size " + map.size() );
            }
        }
    }
}
