import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Queue;
import java.util.regex.Pattern;
import org.eclipse.epf.richtext.IRichText;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusAdapter;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.KeyAdapter;
import org.eclipse.swt.events.KeyEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
public class SpellCheckerDialog extends BaseDialog {
    private SpellChecker spellChecker;
    private Queue<String> misspelledWords = new LinkedList<String>();
    private Text wrongWord;
    private List suggestedWords;
    private Button ignoreButton;
    private Button changeButton;
    private IgnoreButtonListener ignoreButtonListener = new IgnoreButtonListener();
    private ChangeButtonListener changeButtonListener = new ChangeButtonListener();
    private ModifiedChangeButtonListener modifiedChangeButtonListener = new ModifiedChangeButtonListener();
    private ModifiedIgnoreButtonListener modifiedIgnoreButtonListener = new ModifiedIgnoreButtonListener();
    private String oldWrongWord;
    private int currentWord = 1;
    private int totalWords;
    private IRichText richText;
    private String noHTMLString;
    public SpellCheckerDialog ( Shell parent, IRichText richText ) {
        super ( parent );
        setShellStyle ( SWT.DIALOG_TRIM | SWT.MODELESS | getDefaultOrientation() );
        this.richText = richText;
        if ( richText != null ) {
            try {
                spellChecker = new SpellChecker();
                findMisspelledWords();
            } catch ( URISyntaxException e ) {
                MessageDialog.openInformation ( Display.getDefault().getActiveShell(), "Dictionary file for rich text editor not found", e.getMessage() );
                e.printStackTrace();
            } catch ( IOException e ) {
                MessageDialog.openInformation ( Display.getDefault().getActiveShell(), "Dictionary file for rich text editor not found", e.getMessage() );
                e.printStackTrace();
            }
        }
    }
    @Override
    protected Control createDialogArea ( Composite parent ) {
        Composite composite = ( Composite ) super.createDialogArea ( parent );
        composite.setLayout ( new GridLayout ( 2, false ) );
        composite.setLayoutData ( new GridData ( SWT.FILL, SWT.FILL, true, true ) );
        new Label ( composite, SWT.NONE ).setText ( "Not in Dictionary:" );
        new Label ( composite, SWT.NONE );
        wrongWord = new Text ( composite, SWT.MULTI | SWT.BORDER | SWT.WRAP | SWT.V_SCROLL );
        GridData gd = new GridData ( SWT.FILL, SWT.FILL, true, true );
        gd.heightHint = 75;
        gd.minimumHeight = 75;
        wrongWord.setLayoutData ( gd );
        wrongWord.addKeyListener ( new WrongWordKeyListener() );
        wrongWord.addFocusListener ( new WrongWordFocusListener() );
        GridData buttonCompositeGridData = new GridData ( SWT.FILL, SWT.FILL, false, true );
        buttonCompositeGridData.widthHint = 125;
        Composite topButtonActionsComposite = new Composite ( composite, SWT.NONE );
        topButtonActionsComposite.setLayout ( new GridLayout() );
        topButtonActionsComposite.setLayoutData ( buttonCompositeGridData );
        ignoreButton = new Button ( topButtonActionsComposite, SWT.PUSH );
        ignoreButton.setText ( "Ignore" );
        ignoreButton.setLayoutData ( new GridData ( SWT.FILL, SWT.FILL, true, false ) );
        ignoreButton.addSelectionListener ( ignoreButtonListener );
        new Label ( composite, SWT.NONE ).setText ( "Suggestions:" );
        new Label ( composite, SWT.NONE );
        suggestedWords = new List ( composite, SWT.BORDER | SWT.V_SCROLL | SWT.SINGLE );
        suggestedWords.setLayoutData ( new GridData ( SWT.FILL, SWT.FILL, true, true ) );
        Composite bottomButtonActionsComposite = new Composite ( composite, SWT.NONE );
        bottomButtonActionsComposite.setLayout ( new GridLayout() );
        bottomButtonActionsComposite.setLayoutData ( buttonCompositeGridData );
        changeButton = new Button ( bottomButtonActionsComposite, SWT.PUSH );
        changeButton.setText ( "Change" );
        changeButton.setLayoutData ( new GridData ( SWT.FILL, SWT.FILL, true, false ) );
        changeButton.addSelectionListener ( changeButtonListener );
        startSpellChecking();
        return composite;
    }
    private void startSpellChecking() {
        if ( !misspelledWords.isEmpty() ) {
            super.getShell().setText ( "Spell Checker (" + currentWord + "/" + totalWords + ")" );
            String firstWord = misspelledWords.poll();
            HashMap<Integer, String> suggestions = spellChecker.suggestions ( firstWord );
            java.util.List<Integer> sortedKeys = asSortedList ( suggestions.keySet() );
            wrongWord.setText ( firstWord );
            suggestedWords.removeAll();
            for ( Integer i : sortedKeys ) {
                suggestedWords.add ( suggestions.get ( i ) );
            }
            if ( suggestions.size() == 0 ) {
                suggestedWords.add ( "No Suggestions" );
                changeButton.setEnabled ( false );
            } else {
                changeButton.setEnabled ( true );
            }
            suggestedWords.select ( 0 );
            currentWord++;
        } else {
            MessageDialog.openInformation ( Display.getDefault().getActiveShell(), "No Misspellings", "There were no more misspelled words found." );
            super.close();
        }
    }
    private void findMisspelledWords() {
        misspelledWords.clear();
        String[] cleanWords = cleanText ( richText.getText() );
        for ( String w : cleanWords ) {
            System.out.println ( w );
        }
        for ( String word : cleanWords ) {
            if ( !word.equals ( " " ) && !word.isEmpty() ) {
                if ( !spellChecker.isInDictionary ( word ) ) {
                    misspelledWords.add ( word );
                }
            }
        }
        totalWords = misspelledWords.size();
    }
    private String[] cleanText ( String text ) {
        noHTMLString = text.replaceAll ( "\\<.*?>", "" );
        noHTMLString = noHTMLString.replaceAll ( "&.*?;", "" );
        if ( noHTMLString.contains ( "<" ) ) {
            int length = 0;
            while ( noHTMLString.length() != length ) {
                length = htmlTagRemoval ( noHTMLString );
            }
            System.out.println ( "after manual removal noHTMLString: " + noHTMLString );
        }
        noHTMLString = noHTMLString.replaceAll ( "[\\t\\n\\r]", " " );
        String[] allWords = noHTMLString.split ( " " );
        java.util.List<String> cleanedAllWords = new ArrayList<String>();
        for ( String word : allWords ) {
            word = word.trim();
            if ( !containsOnlyLetters ( word ) ) {
                word = removeNonLetters ( word );
            }
            cleanedAllWords.add ( word );
        }
        for ( int i = 0; i < cleanedAllWords.size(); i++ ) {
            cleanedAllWords.set ( i, cleanedAllWords.get ( i ).toLowerCase() );
        }
        return cleanedAllWords.toArray ( new String[cleanedAllWords.size()] );
    }
    private int htmlTagRemoval ( String string ) {
        int i = 0;
        Integer start = null, end = null;
        for ( i = 0; i < string.length(); i++ ) {
            if ( string.toCharArray() [i] == '<' ) {
                start = i;
            }
            if ( start != null && string.toCharArray() [i] == '>' ) {
                end = i;
                noHTMLString = string.replace ( string.subSequence ( start, end + 1 ), "" );
                break;
            }
        }
        return i;
    }
    private boolean containsOnlyLetters ( String s ) {
        if ( Pattern.matches ( "[a-zA-Z]+", s ) ) {
            return true;
        } else {
            return false;
        }
    }
    private String removeNonLetters ( String s ) {
        StringBuffer sb = new StringBuffer();
        for ( int i = 0; i < s.length(); i++ ) {
            if ( Character.isLetter ( s.charAt ( i ) ) || s.charAt ( i ) == '\'' ) {
                sb = sb.append ( s.charAt ( i ) );
            }
        }
        return sb.toString();
    }
    private class WrongWordKeyListener extends KeyAdapter {
        @Override
        public void keyReleased ( KeyEvent e ) {
            suggestedWords.setEnabled ( false );
            ignoreButton.setEnabled ( true );
            ignoreButton.setText ( "Undo Edit" );
            ignoreButton.removeSelectionListener ( ignoreButtonListener );
            ignoreButton.addSelectionListener ( modifiedIgnoreButtonListener );
            changeButton.setEnabled ( true );
            changeButton.removeSelectionListener ( changeButtonListener );
            changeButton.addSelectionListener ( modifiedChangeButtonListener );
            wrongWord.removeKeyListener ( this );
        }
    }
    private class WrongWordFocusListener extends FocusAdapter {
        @Override
        public void focusGained ( FocusEvent e ) {
            oldWrongWord = wrongWord.getText();
            super.focusGained ( e );
        }
    }
    private class IgnoreButtonListener extends SelectionAdapter {
        @Override
        public void widgetSelected ( SelectionEvent e ) {
            startSpellChecking();
        }
    }
    private class ChangeButtonListener extends SelectionAdapter {
        @Override
        public void widgetSelected ( SelectionEvent e ) {
            String newWord = suggestedWords.getSelection() [0];
            String correctedChanges = richText.getText().replace ( wrongWord.getText(), newWord );
            richText.setText ( correctedChanges );
            startSpellChecking();
        }
    }
    private class ModifiedChangeButtonListener extends SelectionAdapter {
        @Override
        public void widgetSelected ( SelectionEvent e ) {
            String newWord = wrongWord.getText();
            String correctedChanges = richText.getText().replace ( oldWrongWord, newWord );
            richText.setText ( correctedChanges );
            ignoreButton.setText ( "Ignore" );
            ignoreButton.removeSelectionListener ( modifiedIgnoreButtonListener );
            ignoreButton.addSelectionListener ( ignoreButtonListener );
            changeButton.removeSelectionListener ( modifiedChangeButtonListener );
            changeButton.addSelectionListener ( changeButtonListener );
            suggestedWords.setEnabled ( true );
            wrongWord.addKeyListener ( new WrongWordKeyListener() );
            startSpellChecking();
        }
    }
    private class ModifiedIgnoreButtonListener extends SelectionAdapter {
        @Override
        public void widgetSelected ( SelectionEvent e ) {
            ignoreButton.setText ( "Ignore" );
            ignoreButton.removeSelectionListener ( modifiedIgnoreButtonListener );
            ignoreButton.addSelectionListener ( ignoreButtonListener );
            changeButton.removeSelectionListener ( modifiedChangeButtonListener );
            changeButton.addSelectionListener ( changeButtonListener );
            suggestedWords.setEnabled ( true );
            wrongWord.setText ( oldWrongWord );
            wrongWord.addKeyListener ( new WrongWordKeyListener() );
            if ( suggestedWords.getItem ( 0 ).equals ( "No Suggestions" ) ) {
                changeButton.setEnabled ( false );
            }
        }
    }
    private static <T extends Comparable<? super T>> java.util.List<T> asSortedList ( Collection<T> c ) {
        java.util.List<T> list = new ArrayList<T> ( c );
        java.util.Collections.sort ( list );
        java.util.Collections.reverse ( list );
        return list;
    }
    @Override
    protected Control createButtonBar ( Composite parent ) {
        return null;
    }
    @Override
    protected Point getInitialSize() {
        Point size = super.getInitialSize();
        size.x = 500;
        size.y = 350;
        return size;
    }
}
