public class ListHelper {
    public static <T> List<T> newList() {
        return new ArrayList<T>();
    }
    public static <T> List<T> newList ( List<T> list ) {
        return new ArrayList<T> ( list );
    }
    public static <T> Difference<T> compare ( List<T> oldList, List<T> newList ) {
        final List<T> removed = newList ( oldList );
        final List<T> added = newList ( newList );
        added.removeAll ( oldList );
        removed.removeAll ( newList );
        return new Difference<T> ( added, removed );
    }
    public static <T extends UniqueIdProvider> UpdateInfo<T> getUpdateInfo ( List<T> oldList, List<T> newList ) {
        Difference<T> difference = compare ( oldList, newList );
        return getUpdateInfo ( difference );
    }
    public static <T extends UniqueIdProvider> UpdateInfo<T> getUpdateInfo ( Difference<T> difference ) {
        if ( difference.isIdentical() ) {
            return new UpdateInfo<T> ( new ArrayList<T>(), new ArrayList<T>(), new ArrayList<UpdateValueContainer<T>>() );
        } else {
            List<T> difAdd = difference.getAdded();
            List<T> difRem = difference.getRemoved();
            List<T> toAdd;
            List<T> toRemove;
            List<UpdateValueContainer<T>> toUpdate = newList();
            if ( difRem.size() == 0 ) {
                toAdd = difAdd;
                toRemove = newList();
            } else if ( difAdd.size() == 0 ) {
                toRemove = difRem;
                toAdd = newList();
            } else {
                toAdd = newList();
                for ( T added : difAdd ) {
                    if ( added != null ) {
                        Object addedId = added.getUniqueId();
                        if ( addedId != null ) {
                            boolean matchFound = false;
                            for ( int i = 0; i < difRem.size() && !matchFound; i++ ) {
                                T removed = difRem.get ( i );
                                if ( removed != null ) {
                                    Object removedId = removed.getUniqueId();
                                    if ( matchFound = addedId.equals ( removedId ) ) {
                                        UpdateValueContainer<T> action = new UpdateValueContainer<T> ( removed, added );
                                        toUpdate.add ( action );
                                        difRem.remove ( removed );
                                    }
                                }
                            }
                            if ( !matchFound ) {
                                toAdd.add ( added );
                            }
                        }
                    }
                }
                difRem.removeAll ( toAdd );
                difRem.removeAll ( toUpdate );
                toRemove = difRem;
            }
            return new UpdateInfo<T> ( toAdd, toRemove, toUpdate );
        }
    }
}
public class Difference<T> {
    private final int addedCount;
    private final int removedCount;
    private final boolean identical;
    private final List<T> added;
    private final List<T> removed;
    Difference ( List<T> added, List<T> removed ) {
        this.added = added;
        this.removed = removed;
        if ( added != null ) {
            this.addedCount = added.size();
        } else {
            throw new IllegalStateException ( "List of added items is null" );
        }
        if ( removed != null ) {
            this.removedCount = removed.size();
        } else {
            throw new IllegalStateException ( "List of removed items is null" );
        }
        this.identical = this.removedCount == 0 && this.addedCount == 0;
    }
    public List<T> getAdded() {
        return added;
    }
    public List<T> getRemoved() {
        return removed;
    }
    public int getAddedCount() {
        return addedCount;
    }
    public int getRemovedCount() {
        return removedCount;
    }
    public boolean isIdentical() {
        return identical;
    }
}
public class UpdateInfo<T> {
    private final List<T> toAdd;
    private final List<T> toRemove;
    private final List<UpdateValueContainer<T>> toUpdate;
    private final boolean updateRequired;
    public UpdateInfo ( List<T> toAdd, List<T> toRemove, List<UpdateValueContainer<T>> toUpdate ) {
        this.toAdd = toAdd;
        this.toRemove = toRemove;
        this.toUpdate = toUpdate;
        this.updateRequired = Eval.notNullOrEmpty ( toAdd )
                              || Eval.notNullOrEmpty ( toRemove )
                              || Eval.notNullOrEmpty ( toUpdate );
    }
    public List<T> getToAdd() {
        return toAdd;
    }
    public List<T> getToRemove() {
        return toRemove;
    }
    public List<UpdateValueContainer<T>> getToUpdate() {
        return toUpdate;
    }
    public boolean isUpdateRequired() {
        return updateRequired;
    }
}
public class UpdateValueContainer<T> {
    private final T oldItem;
    private final T newItem;
    public UpdateValueContainer ( T oldItem, T newItem ) {
        this.oldItem = oldItem;
        this.newItem = newItem;
    }
    public T getOldItem() {
        return oldItem;
    }
    public T getNewItem() {
        return newItem;
    }
    @Override
    public boolean equals ( Object o ) {
        if ( this == o ) {
            return true;
        }
        if ( o == null || getClass() != o.getClass() ) {
            return false;
        }
        UpdateValueContainer that = ( UpdateValueContainer ) o;
        if ( newItem != null ? !newItem.equals ( that.newItem ) : that.newItem != null ) {
            return false;
        }
        if ( oldItem != null ? !oldItem.equals ( that.oldItem ) : that.oldItem != null ) {
            return false;
        }
        return true;
    }
    @Override
    public int hashCode() {
        int result = oldItem != null ? oldItem.hashCode() : 0;
        result = 31 * result + ( newItem != null ? newItem.hashCode() : 0 );
        return result;
    }
}
