public class Track {
    private static Logger log = LoggerFactory.getLogger ( Track.class );
    private final long id = System.nanoTime();
    private int channelCount;
    private int bufferedSeconds = 5;
    private AtomicInteger count = new AtomicInteger ( 0 );
    private ReentrantLock lock;
    private volatile short[] buffer;
    private int capacity = 0;
    private int head = 0;
    private int tail = 0;
    public Track ( int samplingRate, int channelCount ) {
        this.channelCount = channelCount;
        capacity = ( samplingRate * channelCount ) * bufferedSeconds;
        buffer = new short[capacity];
        lock = new ReentrantLock ( true );
    }
    public int getSamplesCount() {
        int i = count.get();
        return i > 0 ? i / channelCount : 0;
    }
    public Short remove() {
        Short sample = null;
        if ( count.get() > 0 ) {
            count.addAndGet ( -1 );
            head = ( head + 1 ) % capacity;
            sample = buffer[head];
        } else {
            log.debug ( "Buffer underflow" );
        }
        return sample;
    }
    public boolean add ( short sample ) {
        boolean result = false;
        if ( ( count.get() + 1 ) < capacity ) {
            count.addAndGet ( 1 );
            tail = ( tail + 1 ) % capacity;
            buffer[tail] = sample;
            result = true;
        } else {
            log.debug ( "Buffer overflow" );
        }
        return result;
    }
    public boolean offer ( short[] samples ) {
        boolean result = false;
        if ( ( count.get() + samples.length ) <= capacity ) {
            pushSamples ( samples );
            result = true;
        }
        return result;
    }
    public void pushSamples ( short[] samples ) {
        log.trace ( "[{}] pushSamples - count: {}", id, samples.length );
        try {
            lock.tryLock ( 10, TimeUnit.MILLISECONDS );
            for ( short sample : samples ) {
                log.trace ( "Position at write: {}", tail );
                if ( !add ( sample ) ) {
                    log.warn ( "Sample could not be added" );
                    break;
                }
            }
        } catch ( InterruptedException e ) {
            log.warn ( "Exception getting samples", e );
        } finally {
            if ( lock.isHeldByCurrentThread() ) {
                lock.unlock();
            }
        }
    }
    public Short popSample ( int channel ) {
        log.trace ( "[{}] popSample - channel: {}", id, channel );
        Short sample = null;
        if ( channel < channelCount ) {
            log.trace ( "Position at read: {}", head );
            try {
                lock.tryLock ( 10, TimeUnit.MILLISECONDS );
                sample = remove();
            } catch ( InterruptedException e ) {
                log.warn ( "Exception getting sample", e );
            } finally {
                if ( lock.isHeldByCurrentThread() ) {
                    lock.unlock();
                }
            }
        }
        return sample;
    }
