import java.math.*;
public final class BigFraction extends Number implements Comparable<BigFraction> {
    private static final long serialVersionUID = 1L;
    private final BigInteger numerator;
    private final BigInteger denominator;
    public final static BigFraction ZERO = new BigFraction ( BigInteger.ZERO, BigInteger.ONE, true );
    public final static BigFraction ONE = new BigFraction ( BigInteger.ONE, BigInteger.ONE, true );
    public BigFraction ( BigInteger numerator, BigInteger denominator ) {
        if ( numerator == null ) {
            throw new IllegalArgumentException ( "Numerator is null" );
        }
        if ( denominator == null ) {
            throw new IllegalArgumentException ( "Denominator is null" );
        }
        if ( denominator.equals ( BigInteger.ZERO ) ) {
            throw new ArithmeticException ( "Divide by zero." );
        }
        if ( denominator.signum() < 0 ) {
            numerator = numerator.negate();
            denominator = denominator.negate();
        }
        BigInteger gcd = numerator.gcd ( denominator );
        this.numerator = numerator.divide ( gcd );
        this.denominator = denominator.divide ( gcd );
    }
    public BigFraction ( BigInteger numerator ) {
        this ( numerator, BigInteger.ONE, true );
    }
    public BigFraction ( long numerator, long denominator ) {
        this ( BigInteger.valueOf ( numerator ), BigInteger.valueOf ( denominator ) );
    }
    public BigFraction ( long numerator ) {
        this ( BigInteger.valueOf ( numerator ), BigInteger.ONE, true );
    }
    public BigFraction ( double d ) {
        if ( Double.isInfinite ( d ) ) {
            throw new IllegalArgumentException ( "double val is infinite" );
        }
        if ( Double.isNaN ( d ) ) {
            throw new IllegalArgumentException ( "double val is NaN" );
        }
        if ( d == 0 ) {
            numerator = BigInteger.ZERO;
            denominator = BigInteger.ONE;
            return;
        }
        final long bits = Double.doubleToLongBits ( d );
        final int sign = ( int ) ( bits >> 63 ) & 0x1;
        final int exponent = ( ( int ) ( bits >> 52 ) & 0x7ff ) - 0x3ff;
        final long mantissa = bits & 0xfffffffffffffL;
        BigInteger tmpNumerator = BigInteger.valueOf ( sign == 0 ? 1 : -1 );
        BigInteger tmpDenominator = BigInteger.ONE;
        if ( exponent >= 0 ) {
            tmpNumerator = tmpNumerator.multiply ( BigInteger.ONE.shiftLeft ( exponent ) );
        } else {
            tmpDenominator = tmpDenominator.multiply ( BigInteger.ONE.shiftLeft ( -exponent ) );
        }
        tmpDenominator = tmpDenominator.multiply ( BigInteger.valueOf ( 0x10000000000000L ) );
        tmpNumerator = tmpNumerator.multiply ( BigInteger.valueOf ( 0x10000000000000L + mantissa ) );
        BigInteger gcd = tmpNumerator.gcd ( tmpDenominator );
        numerator = tmpNumerator.divide ( gcd );
        denominator = tmpDenominator.divide ( gcd );
    }
    public BigFraction ( double numerator, double denominator ) {
        if ( denominator == 0 ) {
            throw new ArithmeticException ( "Divide by zero." );
        }
        BigFraction tmp = new BigFraction ( numerator ).divide ( new BigFraction ( denominator ) );
        this.numerator = tmp.numerator;
        this.denominator = tmp.denominator;
    }
    public BigFraction ( BigDecimal d ) {
        this ( d.scale() < 0 ? d.unscaledValue().multiply ( BigInteger.TEN.pow ( -d.scale() ) ) : d.unscaledValue(),
               d.scale() < 0 ? BigInteger.ONE                                             : BigInteger.TEN.pow ( d.scale() ) );
    }
    public BigFraction ( BigDecimal numerator, BigDecimal denominator ) {
        if ( denominator.equals ( BigDecimal.ZERO ) ) {
            throw new ArithmeticException ( "Divide by zero." );
        }
        BigFraction tmp = new BigFraction ( numerator ).divide ( new BigFraction ( denominator ) );
        this.numerator = tmp.numerator;
        this.denominator = tmp.denominator;
    }
    public BigFraction ( String s ) {
        int slashPos = s.indexOf ( '/' );
        if ( slashPos < 0 ) {
            BigFraction res = new BigFraction ( new BigDecimal ( s ) );
            this.numerator = res.numerator;
            this.denominator = res.denominator;
        } else {
            BigDecimal num = new BigDecimal ( s.substring ( 0, slashPos ) );
            BigDecimal den = new BigDecimal ( s.substring ( slashPos + 1, s.length() ) );
            BigFraction res = new BigFraction ( num, den );
            this.numerator = res.numerator;
            this.denominator = res.denominator;
        }
    }
    public BigFraction add ( BigFraction f ) {
        if ( f == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        return new BigFraction ( numerator.multiply ( f.denominator ).add ( denominator.multiply ( f.numerator ) ),
                                 denominator.multiply ( f.denominator ) );
    }
    public BigFraction add ( BigInteger b ) {
        if ( b == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        return new BigFraction ( numerator.add ( denominator.multiply ( b ) ),
                                 denominator, true );
    }
    public BigFraction add ( long n ) {
        return add ( BigInteger.valueOf ( n ) );
    }
    public BigFraction subtract ( BigFraction f ) {
        if ( f == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        return new BigFraction ( numerator.multiply ( f.denominator ).subtract ( denominator.multiply ( f.numerator ) ),
                                 denominator.multiply ( f.denominator ) );
    }
    public BigFraction subtract ( BigInteger b ) {
        if ( b == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        return new BigFraction ( numerator.subtract ( denominator.multiply ( b ) ),
                                 denominator, true );
    }
    public BigFraction subtract ( long n ) {
        return subtract ( BigInteger.valueOf ( n ) );
    }
    public BigFraction multiply ( BigFraction f ) {
        if ( f == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        return new BigFraction ( numerator.multiply ( f.numerator ), denominator.multiply ( f.denominator ) );
    }
    public BigFraction multiply ( BigInteger b ) {
        if ( b == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        return new BigFraction ( numerator.multiply ( b ), denominator );
    }
    public BigFraction multiply ( long n ) {
        return multiply ( BigInteger.valueOf ( n ) );
    }
    public BigFraction divide ( BigFraction f ) {
        if ( f == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        if ( f.numerator.equals ( BigInteger.ZERO ) ) {
            throw new ArithmeticException ( "Divide by zero" );
        }
        return new BigFraction ( numerator.multiply ( f.denominator ), denominator.multiply ( f.numerator ) );
    }
    public BigFraction divide ( BigInteger b ) {
        if ( b == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        if ( b.equals ( BigInteger.ZERO ) ) {
            throw new ArithmeticException ( "Divide by zero" );
        }
        return new BigFraction ( numerator, denominator.multiply ( b ) );
    }
    public BigFraction divide ( long n ) {
        return divide ( BigInteger.valueOf ( n ) );
    }
    public BigFraction pow ( int exponent ) {
        if ( exponent == 0 ) {
            return BigFraction.ONE;
        } else if ( exponent == 1 ) {
            return this;
        } else if ( exponent < 0 ) {
            return new BigFraction ( denominator.pow ( -exponent ), numerator.pow ( -exponent ), true );
        } else {
            return new BigFraction ( numerator.pow ( exponent ), denominator.pow ( exponent ), true );
        }
    }
    public BigFraction reciprocal() {
        if ( this.numerator.equals ( BigInteger.ZERO ) ) {
            throw new ArithmeticException ( "Divide by zero" );
        }
        return new BigFraction ( denominator, numerator, true );
    }
    public BigFraction complement() {
        return new BigFraction ( denominator.subtract ( numerator ), denominator, true );
    }
    public BigFraction negate() {
        return new BigFraction ( numerator.negate(), denominator, true );
    }
    public int signum() {
        return numerator.signum();
    }
    public BigFraction abs() {
        return ( signum() < 0 ? negate() : this );
    }
    public String toString() {
        return numerator.toString() + "/" + denominator.toString();
    }
    public boolean equals ( Object o ) {
        if ( ! ( o instanceof BigFraction ) ) {
            return false;
        }
        BigFraction f = ( BigFraction ) o;
        return numerator.equals ( f.numerator ) && denominator.equals ( f.denominator );
    }
    public int hashCode() {
        return ( 31 + numerator.hashCode() ) * 31 + denominator.hashCode();
    }
    public int compareTo ( BigFraction f ) {
        if ( f == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        if ( signum() != f.signum() ) {
            return signum() - f.signum();
        }
        if ( denominator.equals ( f.denominator ) ) {
            return numerator.compareTo ( f.numerator );
        }
        return numerator.multiply ( f.denominator ).compareTo ( denominator.multiply ( f.numerator ) );
    }
    public BigFraction min ( BigFraction f ) {
        if ( f == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        return ( this.compareTo ( f ) <= 0 ? this : f );
    }
    public BigFraction max ( BigFraction f ) {
        if ( f == null ) {
            throw new IllegalArgumentException ( "Null argument" );
        }
        return ( this.compareTo ( f ) >= 0 ? this : f );
    }
    public static BigFraction random() {
        return new BigFraction ( Math.random() );
    }
    public final BigInteger getNumerator() {
        return numerator;
    }
    public final BigInteger getDenominator() {
        return denominator;
    }
    public byte   byteValue()   {
        return ( byte ) Math.max ( Byte.MIN_VALUE,    Math.min ( Byte.MAX_VALUE,    longValue() ) );
    }
    public short  shortValue()  {
        return ( short ) Math.max ( Short.MIN_VALUE,   Math.min ( Short.MAX_VALUE,   longValue() ) );
    }
    public int    intValue()    {
        return ( int )  Math.max ( Integer.MIN_VALUE, Math.min ( Integer.MAX_VALUE, longValue() ) );
    }
    public long   longValue()   {
        return Math.round ( doubleValue() );
    }
    public float  floatValue()  {
        return ( float ) doubleValue();
    }
    public double doubleValue() {
        return toBigDecimal ( 18 ).doubleValue();
    }
    public BigDecimal toBigDecimal() {
        int twos = denominator.getLowestSetBit();
        BigInteger tmpDen = denominator.shiftRight ( twos );
        final BigInteger FIVE = BigInteger.valueOf ( 5 );
        int fives = 0;
        BigInteger[] divMod = null;
        while ( BigInteger.ZERO.equals ( ( divMod = tmpDen.divideAndRemainder ( FIVE ) ) [1] ) ) {
            fives++;
            tmpDen = divMod[0];
        }
        if ( BigInteger.ONE.equals ( tmpDen ) ) {
            BigInteger unscaled = numerator;
            int scale = Math.max ( twos, fives );
            if ( twos < fives ) {
                unscaled = unscaled.shiftLeft ( fives - twos );
            } else if ( fives < twos ) {
                unscaled = unscaled.multiply ( FIVE.pow ( twos - fives ) );
            }
            return new BigDecimal ( unscaled, scale );
        }
        final double LG10 = 3.321928094887362;
        int precision = Math.max ( numerator.bitLength(), denominator.bitLength() );
        precision = ( int ) Math.ceil ( precision / LG10 );
        if ( precision < 18 ) {
            precision = 18;
        }
        return toBigDecimal ( precision );
    }
    public BigDecimal toBigDecimal ( int precision ) {
        return new BigDecimal ( numerator ).divide ( new BigDecimal ( denominator ), new MathContext ( precision, RoundingMode.HALF_EVEN ) );
    }
    private BigFraction ( BigInteger numerator, BigInteger denominator, boolean throwaway ) {
        if ( denominator.signum() < 0 ) {
            this.numerator = numerator.negate();
            this.denominator = denominator.negate();
        } else {
            this.numerator = numerator;
            this.denominator = denominator;
        }
    }
}
