import java.io.*;
import java.util.*;
import java.util.zip.*;
public class ZipNode {
    private ZipNode parent;
    private Map<String, ZipNode> children;
    private boolean directory;
    private ZipEntry entry;
    private ZipFile file;
    private ZipNode ( ZipFile f, ZipEntry entry ) {
        this.file = f;
        this.entry = entry;
        if ( entry == null || entry.isDirectory() ) {
            directory = true;
            children = new LinkedHashMap<String, ZipNode>();
        } else {
            directory = false;
            children = Collections.emptyMap();
        }
    }
    public String getName() {
        if ( entry == null ) {
            return "/";
        }
        String longName = entry.getName();
        return longName.substring ( longName.lastIndexOf ( '/',
                                    longName.length() - 2 ) + 1 );
    }
    public ZipEntry getEntry() {
        return entry;
    }
    public ZipFile getZipFile() {
        return file;
    }
    public boolean isDirectory() {
        return directory;
    }
    public ZipNode getParent() {
        return parent;
    }
    public Map<String, ZipNode> getChildren() {
        return Collections.unmodifiableMap ( children );
    }
    public InputStream openStream()
    throws IOException {
        return file.getInputStream ( entry );
    }
    public String toString() {
        return "ZipNode [" + entry.getName() + "] in [" + file.getName() + "]";
    }
    public static ZipNode fromZipFile ( ZipFile zf ) {
        return new ZipFileReader ( zf ).process();
    }
    private static class ZipFileReader {
        private ZipFile file;
        private Map<String, ZipNode> collected;
        private ZipNode root;
        ZipFileReader ( ZipFile f ) {
            this.file = f;
            this.collected = new HashMap<String, ZipNode>();
            collected.put ( "", root );
            root = new ZipNode ( f, null );
        }
        ZipNode process() {
            for ( Enumeration<? extends ZipEntry> entries = file.entries();
                    entries.hasMoreElements(); ) {
                this.addEntry ( entries.nextElement() );
            }
            return root;
        }
        private ZipNode addEntry ( ZipEntry entry ) {
            String name = entry.getName();
            ZipNode node = collected.get ( name );
            if ( node != null ) {
                return node;
            }
            node = new ZipNode ( file, entry );
            collected.put ( name, node );
            this.findParent ( node );
            return node;
        }
        private void findParent ( ZipNode node ) {
            String nodeName = node.entry.getName();
            int slashIndex = nodeName.lastIndexOf ( '/', nodeName.length() - 2 );
            if ( slashIndex < 0 ) {
                connectParent ( root, node, nodeName );
                return;
            }
            String parentName = nodeName.substring ( 0, slashIndex + 1 );
            ZipNode parent = addEntry ( file.getEntry ( parentName ) );
            connectParent ( parent, node, nodeName.substring ( slashIndex + 1 ) );
        }
        private void connectParent ( ZipNode parent, ZipNode child,
                                     String childName ) {
            child.parent = parent;
            parent.children.put ( childName, child );
        }
    }
    public static void main ( String[] params )
    throws IOException {
        if ( params.length < 1 ) {
            System.err.println ( "Invocation:  java ZipNode zipFile.zip" );
            return;
        }
        ZipFile file = new ZipFile ( params[0] );
        ZipNode root = ZipNode.fromZipFile ( file );
        file.close();
        root.printTree ( "", " ", "" );
    }
    private void printTree ( String prefix,
                             String self,
                             String sub ) {
        System.out.println ( prefix + self + this.getName() );
        String subPrefix = prefix + sub;
        String nextSelf = " ├─ ";
        String nextSub =  " │ ";
        Iterator<ZipNode> iterator =
            this.getChildren().values().iterator();
        while ( iterator.hasNext() ) {
            ZipNode child = iterator.next();
            if ( !iterator.hasNext() ) {
                nextSelf = " ╰─ ";
                nextSub =  "   ";
            }
            child.printTree ( subPrefix, nextSelf, nextSub );
        }
    }
}
