public class RGBImageLoader {
    private byte[] tempBuffer_;
    private boolean fastLoading_;
    public RGBImageLoader() {
        fastLoading_ = canUseFastLoadingTechnique();
    }
    private boolean canUseFastLoadingTechnique() {
        GraphicsConfiguration gc = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
        BufferedImage image = gc.createCompatibleImage ( 100, 100, Transparency.TRANSLUCENT );
        if ( image.getType() != BufferedImage.TYPE_INT_ARGB ) {
            return false;
        }
        WritableRaster raster = image.getRaster();
        if ( ! ( raster instanceof IntegerInterleavedRaster ) ) {
            return false;
        }
        if ( ! ( raster.getDataBuffer() instanceof DataBufferInt ) ) {
            return false;
        }
        if ( ! ( image.getColorModel() instanceof DirectColorModel ) ) {
            return false;
        }
        DirectColorModel colorModel = ( DirectColorModel ) image.getColorModel();
        if ( ! ( colorModel.getColorSpace() instanceof ICC_ColorSpace ) ||
                colorModel.getNumComponents() != 4 ||
                colorModel.getAlphaMask() != 0xff000000 ||
                colorModel.getRedMask() != 0xff0000 ||
                colorModel.getGreenMask() != 0xff00 ||
                colorModel.getBlueMask() != 0xff ) {
            return false;
        }
        if ( raster.getNumBands() != 4 ||
                raster.getNumDataElements() != 1 ||
                ! ( raster.getSampleModel() instanceof SinglePixelPackedSampleModel ) ) {
            return false;
        }
        return true;
    }
    public BufferedImage loadImage ( File file, int width, int height, long imageOffset ) throws IOException {
        if ( fastLoading_ ) {
            return loadImageUsingFastTechnique ( file, width, height, imageOffset );
        } else {
            return loadImageUsingCompatibleTechnique ( file, width, height, imageOffset );
        }
    }
    private BufferedImage loadImageUsingFastTechnique ( File file, int width, int height, long imageOffset ) throws IOException {
        int sizeBytes = width * height * 3;
        if ( tempBuffer_ == null || tempBuffer_.length < sizeBytes ) {
            tempBuffer_ = new byte[sizeBytes];
        }
        RandomAccessFile raf = null;
        try {
            raf = new RandomAccessFile ( file, "r" );
            raf.seek ( imageOffset );
            int bytesRead = raf.read ( tempBuffer_, 0, sizeBytes );
            if ( bytesRead != sizeBytes ) {
                throw new IOException ( "Invalid byte count. Should be " + sizeBytes + " not " + bytesRead );
            }
            GraphicsConfiguration gc = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
            BufferedImage image = gc.createCompatibleImage ( width, height, Transparency.TRANSLUCENT );
            WritableRaster raster = image.getRaster();
            DataBufferInt dataBuffer = ( DataBufferInt ) raster.getDataBuffer();
            addAlphaChannel ( tempBuffer_, sizeBytes, dataBuffer.getData() );
            return image;
        } finally {
            try {
                if ( raf != null ) {
                    raf.close();
                }
            } catch ( Exception ex ) {
            }
        }
    }
    private BufferedImage loadImageUsingCompatibleTechnique ( File file, int width, int height, long imageOffset ) throws IOException {
        int sizeBytes = width * height * 3;
        RandomAccessFile raf = null;
        try {
            raf = new RandomAccessFile ( file, "r" );
            raf.seek ( imageOffset );
            DataBufferByte dataBuffer = new DataBufferByte ( sizeBytes );
            byte[] bytes = dataBuffer.getData();
            int bytesRead = raf.read ( bytes, 0, sizeBytes );
            if ( bytesRead != sizeBytes ) {
                throw new IOException ( "Invalid byte count. Should be " + sizeBytes + " not " + bytesRead );
            }
            WritableRaster raster = Raster.createInterleavedRaster ( dataBuffer,
                                    width,
                                    height,
                                    width * 3,
                                    3,
                                    new int[] {0, 1, 2},
                                    null );
            ColorModel colorModel = new ComponentColorModel ( ColorSpace.getInstance ( ColorSpace.CS_sRGB ),
                    new int[] {8, 8, 8},
                    false,
                    false,
                    ComponentColorModel.OPAQUE, DataBuffer.TYPE_BYTE );
            BufferedImage loadImage = new BufferedImage ( colorModel, raster, false, null );
            BufferedImage image = createCompatibleImage ( loadImage );
            return image;
        } finally {
            try {
                if ( raf != null ) {
                    raf.close();
                }
            } catch ( Exception ex ) {
            }
        }
    }
    private BufferedImage createCompatibleImage ( BufferedImage image ) {
        GraphicsConfiguration gc = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration();
        BufferedImage newImage = gc.createCompatibleImage ( image.getWidth(), image.getHeight(), Transparency.TRANSLUCENT );
        Graphics2D g = newImage.createGraphics();
        g.drawImage ( image, 0, 0, null );
        g.dispose();
        return newImage;
    }
    private void addAlphaChannel ( byte[] rgbBytes, int bytesLen, int[] argbInts ) {
        for ( int i = 0, j = 0; i < bytesLen; i += 3, j++ ) {
            argbInts[j] = ( ( byte ) 0xff ) << 24 |
                          ( rgbBytes[i] << 16 ) & ( 0xff0000 ) |
                          ( rgbBytes[i + 1] << 8 ) & ( 0xff00 ) |
                          ( rgbBytes[i + 2] ) & ( 0xff );
        }
    }
}
