import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import javax.crypto.Cipher;
import android.util.Log;
public final class XmlSigner {
    private static final String LOG_TAG = "XmlSigner";
    private static final byte[] DER_SHA1_DIGEST_IDENTIFIER = new byte[] {48, 33, 48, 9, 6, 5, 43, 14,
            3, 2, 26, 5, 0, 4, 20
                                                                        };
    private static final char[] CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/".toCharArray();
    private XmlSigner() {
    }
    public static String generateXmlSignature ( String xml, PrivateKey privateKey ) throws Throwable {
        try {
            int soapBodyStart = xml.indexOf ( "<soap:Body>" );
            int soapBodyEnd = xml.indexOf ( "</soap:Body>" );
            String bodyXml = xml.substring ( soapBodyStart + 12, soapBodyEnd - 1 );
            byte[] xmlBytes = bodyXml.getBytes ( "UTF-8" );
            byte[] xmlDigestBytes = sha1Digest ( xmlBytes );
            String base64XmlDigest = base64encode ( xmlDigestBytes, false );
            String signedInfo = "<ds:SignedInfo xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\">"
                                + "<ds:CanonicalizationMethod Algorithm=\"http://www.w3.org/2001/10/xml-exc-c14n#\">"
                                + "</ds:CanonicalizationMethod><ds:SignatureMethod Algorithm=\"http://www.w3.org/2000/09/xmldsig#rsa-sha1\">"
                                + "</ds:SignatureMethod><ds:Reference URI=\"\"><ds:Transforms>"
                                + "<ds:Transform Algorithm=\"http://www.w3.org/2002/06/xmldsig-filter2\">"
                                + "<ds:XPath xmlns:ds=\"http://www.w3.org/2002/06/xmldsig-filter2\" Filter=\"intersect\">"
                                + "/soap:Envelope/soap:Body/*</ds:XPath></ds:Transform>"
                                + "<ds:Transform xmlns:ds=\"http://www.w3.org/2002/06/xmldsig-filter2\" "
                                + "Algorithm=\"http://www.w3.org/2001/10/xml-exc-c14n#\"></ds:Transform>"
                                + "</ds:Transforms><ds:DigestMethod Algorithm=\"http://www.w3.org/2000/09/xmldsig#sha1\">"
                                + "</ds:DigestMethod><ds:DigestValue>" + base64XmlDigest
                                + "</ds:DigestValue></ds:Reference></ds:SignedInfo>";
            byte[] signedInfoBytes = signedInfo.getBytes ( "UTF-8" );
            byte[] signedInfoSha1Digest = sha1Digest ( signedInfoBytes );
            byte[] signedInfoDerSha1Digest = mergeArrays ( DER_SHA1_DIGEST_IDENTIFIER, signedInfoSha1Digest );
            Cipher cipher = Cipher.getInstance ( "RSA/ECB/PKCS1PADDING" );
            cipher.init ( Cipher.ENCRYPT_MODE, privateKey );
            byte[] signatureBytes = cipher.doFinal ( signedInfoDerSha1Digest );
            String base64RsaSignature = base64encode ( signatureBytes, true );
            String signature = "<wsse:Security xmlns:wsse=\"http://docs.oasis-open.org/wss/2004/01/oasis"
                               + "-200401-wss-wssecurity-secext-1.0.xsd\">"
                               + "<ds:Signature xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\">" + signedInfo
                               + "<ds:SignatureValue>" + base64RsaSignature
                               + "</ds:SignatureValue></ds:Signature></wsse:Security>";
            return signature;
        } catch ( Throwable e ) {
            Log.e ( LOG_TAG, "Error generating signature for XML", e );
            throw e;
        }
    }
    public static byte[] mergeArrays ( byte[] array1, byte[] array2 ) {
        byte[] temp = new byte[array1.length + array2.length];
        System.arraycopy ( array1, 0, temp, 0, array1.length );
        System.arraycopy ( array2, 0, temp, array1.length, array2.length );
        return temp;
    }
    public static byte[] sha1Digest ( byte[] data ) {
        MessageDigest mdSha1 = null;
        try {
            mdSha1 = MessageDigest.getInstance ( "SHA-1" );
        } catch ( NoSuchAlgorithmException e1 ) {
            Log.e ( LOG_TAG, "Error initializing SHA1 message digest" );
        }
        mdSha1.update ( data );
        byte[] sha1hash = mdSha1.digest();
        return sha1hash;
    }
    public static byte[] sha256Digest ( byte[] data ) {
        MessageDigest mdSha256 = null;
        try {
            mdSha256 = MessageDigest.getInstance ( "SHA-256" );
        } catch ( NoSuchAlgorithmException e1 ) {
            Log.e ( LOG_TAG, "Error initializing SHA1 message digest" );
        }
        mdSha256.update ( data );
        byte[] sha256hash = mdSha256.digest();
        return sha256hash;
    }
    public static String base64encode ( byte[] data, boolean wrapLines ) {
        int length = data.length;
        StringBuilder sb = new StringBuilder ( length * 3 / 2 );
        int end = length - 3;
        int i = 0;
        int n = 0;
        while ( i <= end ) {
            int d = ( ( ( ( int ) data[i] ) & 0x0ff ) << 16 ) | ( ( ( ( int ) data[i + 1] ) & 0x0ff ) << 8 )
                    | ( ( ( int ) data[i + 2] ) & 0x0ff );
            sb.append ( CHARS[ ( d >> 18 ) & 63] );
            sb.append ( CHARS[ ( d >> 12 ) & 63] );
            sb.append ( CHARS[ ( d >> 6 ) & 63] );
            sb.append ( CHARS[d & 63] );
            i += 3;
            if ( n++ >= 14 ) {
                n = 0;
                if ( wrapLines ) {
                    sb.append ( "\r\n" );
                }
            }
        }
        if ( i == length - 2 ) {
            int d = ( ( ( ( int ) data[i] ) & 0x0ff ) << 16 ) | ( ( ( ( int ) data[i + 1] ) & 255 ) << 8 );
            sb.append ( CHARS[ ( d >> 18 ) & 63] );
            sb.append ( CHARS[ ( d >> 12 ) & 63] );
            sb.append ( CHARS[ ( d >> 6 ) & 63] );
            sb.append ( "=" );
        } else if ( i == length - 1 ) {
            int d = ( ( ( int ) data[i] ) & 0x0ff ) << 16;
            sb.append ( CHARS[ ( d >> 18 ) & 63] );
            sb.append ( CHARS[ ( d >> 12 ) & 63] );
            sb.append ( "==" );
        }
        return sb.toString();
    }
}
