package com.jimtough;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Collections;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang.Validate;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import de.micromata.opengis.kml.v_2_2_0.Kml;
public final class KMZPackager {
    private final static Logger logger =
        LoggerFactory.getLogger ( KMZPackager.class );
    public static abstract class DataSource {
        protected String archivedFileName;
        public abstract void writeToStream ( ZipOutputStream zipOutputStream )
        throws IOException;
    }
    public static final class FileDataSource extends DataSource {
        private File sourceFile;
        public FileDataSource (
            File sourceFile,
            String archivedFileName )
        throws IllegalArgumentException {
            Validate.notNull ( sourceFile );
            Validate.notEmpty ( archivedFileName );
            this.sourceFile = sourceFile;
            this.archivedFileName = archivedFileName;
        }
        @Override
        public void writeToStream ( ZipOutputStream zipOutputStream )
        throws IOException {
            Validate.notNull ( zipOutputStream );
            if ( !sourceFile.exists() ) {
                throw new IllegalArgumentException (
                    "File referenced in parameter [" +
                    sourceFile.getAbsolutePath() + "] does not exist" );
            }
            FileInputStream fis = new FileInputStream ( sourceFile );
            if ( logger.isDebugEnabled() ) {
                logger.debug ( "Adding file to KMZ archive" +
                               " | archive name: " + archivedFileName +
                               " | original name: " +
                               sourceFile.getCanonicalPath() );
            }
            ZipEntry entry = new ZipEntry ( archivedFileName );
            zipOutputStream.putNextEntry ( entry );
            try {
                IOUtils.copy ( fis, zipOutputStream );
            } finally {
                fis.close();
            }
        }
    }
    public static final class KMLDataSource extends DataSource {
        private Kml kml;
        public KMLDataSource (
            Kml kml,
            String archivedFileName )
        throws IllegalArgumentException {
            Validate.notNull ( kml );
            Validate.notEmpty ( archivedFileName );
            this.kml = kml;
            this.archivedFileName = archivedFileName;
        }
        @Override
        public void writeToStream ( ZipOutputStream zipOutputStream ) throws IOException {
            Validate.notNull ( zipOutputStream );
            ZipEntry entry = new ZipEntry ( archivedFileName );
            zipOutputStream.putNextEntry ( entry );
            if ( logger.isDebugEnabled() ) {
                logger.debug ( "Marshalling KML to KMZ archive" +
                               " | archive name: " + archivedFileName );
            }
            kml.marshal ( zipOutputStream );
        }
    }
    public static final class StreamDataSource extends DataSource {
        private InputStream inputStream;
        public StreamDataSource (
            InputStream inputStream,
            String archivedFileName )
        throws IllegalArgumentException {
            Validate.notNull ( inputStream );
            Validate.notEmpty ( archivedFileName );
            this.inputStream = inputStream;
            this.archivedFileName = archivedFileName;
        }
        @Override
        public void writeToStream ( ZipOutputStream zipOutputStream ) throws IOException {
            Validate.notNull ( zipOutputStream );
            ZipEntry entry = new ZipEntry ( archivedFileName );
            zipOutputStream.putNextEntry ( entry );
            if ( logger.isDebugEnabled() ) {
                logger.debug ( "Copying KML from stream to KMZ archive" +
                               " | archive name: " + archivedFileName );
            }
            try {
                IOUtils.copy ( inputStream, zipOutputStream );
            } finally {
                inputStream.close();
            }
        }
    }
    public void packageAsKMZ (
        OutputStream os,
        DataSource kmlDataSource,
        List<FileDataSource> supplementaryFileList )
    throws RuntimeException {
        ZipOutputStream zipOutputStream = null;
        boolean isExceptionThrown = false;
        Exception caughtException = null;
        List<FileDataSource> supplFileList = supplementaryFileList;
        try {
            Validate.notNull ( os, "os parameter cannot be null" );
            Validate.notNull ( kmlDataSource,
                               "kmlFileDataSource parameter cannot be null" );
            if ( supplFileList == null ) {
                supplFileList = Collections.emptyList();
            }
            if ( logger.isDebugEnabled() ) {
                logger.debug (
                    "Creating KMZ archive" +
                    " | supplementary files: " + supplFileList.size() );
            }
            zipOutputStream = new ZipOutputStream ( new BufferedOutputStream ( os ) );
            Validate.isTrue (
                kmlDataSource.archivedFileName.endsWith ( ".kml" ),
                "KML archived file name must end with .kml" );
            kmlDataSource.writeToStream ( zipOutputStream );
            if ( logger.isDebugEnabled() ) {
                logger.debug ( "Adding supplementary files to KMZ archive" +
                               " | archive name: " );
            }
            for ( FileDataSource ds : supplFileList ) {
                Validate.isTrue (
                    !ds.archivedFileName.endsWith ( ".kml" ),
                    "Not legal to include .kml files in supplementary list" );
                ds.writeToStream ( zipOutputStream );
            }
            zipOutputStream.flush();
            zipOutputStream.close();
            logger.info ( "KMZ archive created successfully" );
        } catch ( IOException e ) {
            isExceptionThrown = true;
            caughtException = e;
            logger.error ( "IOException while creating ZIP stream" );
        } catch ( IllegalArgumentException e ) {
            isExceptionThrown = true;
            caughtException = e;
        } catch ( RuntimeException e ) {
            isExceptionThrown = true;
            caughtException = e;
        } finally {
            if ( isExceptionThrown ) {
                try {
                    if ( zipOutputStream != null ) {
                        zipOutputStream.close();
                    }
                } catch ( IOException ioe ) {
                }
                throw new RuntimeException ( caughtException );
            }
        }
    }
}
