public class Container<T> implements Iterable<T> {
    final int capacity;
    AtomicReference<Node<T>> head = new AtomicReference<Node<T>>();
    public Container ( int capacity ) {
        this.capacity = capacity;
        Node<T> h = new Node<T>();
        Node<T> it = h;
        for ( int i = 0; i < capacity - 1; i++ ) {
            it.next = new Node<T>();
            it = it.next;
        }
        it.next = h;
        head.set ( h );
    }
    public void clear() {
        Node<T> it = head.get();
        for ( int i = 0; i < capacity; i++ ) {
            it.element = null;
            it.free.set ( true );
            it = it.next;
        }
        resetStats();
    }
    public Node<T> add ( T element ) {
        return getFree().attach ( element );
    }
    private Node<T> getFree() {
        Node<T> freeNode = head.get();
        int skipped = 0;
        while ( skipped < capacity && !freeNode.free.compareAndSet ( true, false ) ) {
            skipped += 1;
            freeNode = freeNode.next;
        }
        if ( skipped < capacity ) {
            head.set ( freeNode.next );
        } else {
            throw new IllegalStateException ( "Capacity exhausted." );
        }
        return freeNode;
    }
    public void remove ( Node<T> it, T element ) {
        it.detach ( element );
        if ( !it.free.compareAndSet ( false, true ) ) {
            throw new IllegalStateException ( "Freeing a freed node." );
        }
    }
    public static class Node<T> {
        private T element;
        private AtomicBoolean free = new AtomicBoolean ( true );
        private Node<T> next;
        private Node() {
            element = null;
        }
        public Node<T> attach ( T element ) {
            if ( this.element == null ) {
                this.element = element;
            } else {
                throw new IllegalArgumentException ( "There is already an element attached." );
            }
            return this;
        }
        public Node<T> detach ( T element ) {
            if ( this.element == element ) {
                this.element = null;
            } else {
                throw new IllegalArgumentException ( "Removal of wrong element." );
            }
            return this;
        }
        @Override
        public String toString() {
            return element != null ? element.toString() : "null";
        }
    }
    public Iterator<T> iterator() {
        return new UsedNodesIterator<T> ( this );
    }
    private static class UsedNodesIterator<T> implements Iterator<T> {
        Node<T> it;
        int limit = 0;
        T next = null;
        public UsedNodesIterator ( Container<T> c ) {
            it = c.head.get();
            limit = c.capacity;
        }
        public boolean hasNext() {
            if ( next == null ) {
                while ( limit > 0 && it.free.get() == true ) {
                    it = it.next;
                    limit -= 1;
                }
                if ( limit != 0 ) {
                    next = it.element;
                }
            }
            return next != null;
        }
        public T next() {
            T n = null;
            if ( hasNext () ) {
                n = next;
                next = null;
                it = it.next;
                limit -= 1;
            } else {
                throw new NoSuchElementException ();
            }
            return n;
        }
        public void remove() {
            throw new UnsupportedOperationException ( "Not supported." );
        }
    }
    @Override
    public String toString() {
        StringBuilder s = new StringBuilder();
        Separator comma = new Separator ( "," );
        int usedCount = 0;
        int freeCount = 0;
        Node<T> it = head.get();
        int count = 0;
        s.append ( "[" );
        while ( count < capacity ) {
            if ( it.free.get() == false ) {
                T e = it.element;
                if ( e != null ) {
                    s.append ( comma.sep() ).append ( e.toString() );
                    usedCount += 1;
                } else {
                    freeCount += 1;
                }
            } else {
                freeCount += 1;
            }
            it = it.next;
            count += 1;
        }
        s.append ( "]" ).append ( usedCount ).append ( "+" ).append ( freeCount );
        if ( usedCount + freeCount != capacity ) {
            s.append ( "?" );
        }
        return s.toString();
    }
}
